##
# File:  WorkManagerWebApp.py
# Date:  23-Apr-2015
#
# Updates:
#  10-Dec-2024 zf  added refreshing "latency of servers" report
##
"""
Chemeditor web request and response processing modules.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle

import datetime
import os
import subprocess
import sys
import time
import traceback

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.apps.wf_engine.engine.WFEapplications import reRunWorkflow, getPicklePath
from wwpdb.apps.workmanager.db_access.DBLoader import DBLoader
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.depict.DepictBase import DepictBase
from wwpdb.apps.workmanager.depict.DepictContent import DepictContent
from wwpdb.apps.workmanager.depict.DepictGroup import DepictGroup
from wwpdb.apps.workmanager.depict.DepictLevel1 import DepictLevel1
from wwpdb.apps.workmanager.depict.DepictOther import DepictOther
from wwpdb.apps.workmanager.depict.DepictSnapShot import DepictSnapShot
from wwpdb.apps.workmanager.depict.ReadConFigFile import ReadConFigFile, loadPickleFile
from wwpdb.apps.workmanager.depict.SearchUtil import SearchUtil
from wwpdb.apps.workmanager.depict.ServerInfoUtil import ServerInfoUtil
from wwpdb.apps.workmanager.file_access.AnnotAssignUtil import AnnotAssignUtil
from wwpdb.apps.workmanager.file_access.CopyFileToAutoGroup import CopyFileToAutoGroup
from wwpdb.apps.workmanager.file_access.LogFileUtil import LogFileUtil
from wwpdb.apps.workmanager.file_access.MileStoneFile import MileStoneFile
from wwpdb.apps.workmanager.task_access.CifChecker import CifChecker
from wwpdb.apps.workmanager.task_access.LigandFinder import LigandFinder
from wwpdb.apps.workmanager.task_access.MetaDataEditor import MetaDataEditor
from wwpdb.apps.workmanager.task_access.MetaDataMerger import MetaDataMerger
from wwpdb.apps.workmanager.task_access.PdbFileGenerator import PdbFileGenerator
from wwpdb.apps.workmanager.task_access.SequenceMerger import SequenceMerger
from wwpdb.apps.workmanager.task_access.StatusUpdater import StatusUpdater
from wwpdb.utils.detach.DetachUtils import DetachUtils
from wwpdb.io.locator.PathInfo import PathInfo
from wwpdb.utils.session.WebRequest import InputRequest, ResponseContent
from wwpdb.utils.session.WebUploadUtils import WebUploadUtils
from wwpdb.utils.config.ProjectVersionInfo import ProjectVersionInfo
from wwpdb.apps.msgmodule.util.AutoMessage import AutoMessage
#


class WorkManagerWebApp(object):
    """Handle request and response object processing for release module web application.

    """
    def __init__(self, parameterDict=None, verbose=False, log=sys.stderr, siteId="WWPDB_DEV"):
        """
        Create an instance of `WorkManagerWebApp` to manage a release module web request.

         :param `parameterDict`: dictionary storing parameter information from the web request.
             Storage model for GET and POST parameter data is a dictionary of lists.
         :param `verbose`:  boolean flag to activate verbose logging.
         :param `log`:      stream for logging.

        """
        if parameterDict is None:
            parameterDict = {}
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False
        self.__siteId = siteId
        self.__cI = ConfigInfo(self.__siteId)
        self.__topPath = self.__cI.get('SITE_WEB_APPS_TOP_PATH')
        #

        if type(parameterDict) == dict:  # noqa: E721
            self.__myParameterDict = parameterDict
        else:
            self.__myParameterDict = {}

        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebApp.__init() - REQUEST STARTING ------------------------------------\n")
            self.__lfh.write("+WorkManagerWebApp.__init() - dumping input parameter dictionary \n")
            self.__lfh.write("%s" % (''.join(self.__dumpRequest())))

        self.__reqObj = InputRequest(self.__myParameterDict, verbose=self.__verbose, log=self.__lfh)

        self.__topSessionPath = self.__cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH')
        self.__templatePath = os.path.join(self.__topPath, "htdocs", "wfm", "templates")
        #
        self.__reqObj.setValue("TopSessionPath", self.__topSessionPath)
        self.__reqObj.setValue("TemplatePath", self.__templatePath)
        self.__reqObj.setValue("TopPath", self.__topPath)
        self.__reqObj.setValue("WWPDB_SITE_ID", self.__siteId)
        self.__reqObj.setValue("current_dep_url", self.__cI.get('SITE_CURRENT_DEP_URL'))
        os.environ["WWPDB_SITE_ID"] = self.__siteId  # EP: Why is this being set?
        #
        project_version = ProjectVersionInfo(self.__siteId).getVersion()
        if project_version:
            self.__reqObj.setValue("DAVersion", project_version)
        #
        self.__reqObj.setReturnFormat(return_format="html")
        #
        if (self.__verbose):
            self.__lfh.write("-----------------------------------------------------\n")
            self.__lfh.write("+WorkManagerWebApp.__init() Leaving _init with request contents\n")
            self.__reqObj.printIt(ofh=self.__lfh)
            self.__lfh.write("---------------WorkManagerWebApp - done -------------------------------\n")
            self.__lfh.flush()

    def doOp(self):
        """ Execute request and package results in response dictionary.

        :Returns:
             A dictionary containing response data for the input request.
             Minimally, the content of this dictionary will include the
             keys: CONTENT_TYPE and REQUEST_STRING.
        """
        stw = WorkManagerWebAppWorker(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC = stw.doOp()
        if (self.__debug):
            rqp = self.__reqObj.getRequestPath()
            self.__lfh.write("+WorkManagerWebApp.doOp() operation %s\n" % rqp)
            self.__lfh.write("+WorkManagerWebApp.doOp() return format %s\n" % self.__reqObj.getReturnFormat())
            if rC is not None:
                self.__lfh.write("%s" % (''.join(rC.dump())))
            else:
                self.__lfh.write("+WorkManagerWebApp.doOp() return object is empty\n")

        #
        # Package return according to the request return_format -
        #
        return rC.get()

    def __dumpRequest(self):
        """Utility method to format the contents of the internal parameter dictionary
           containing data from the input web request.

           :Returns:
               ``list`` of formatted text lines
        """
        retL = []
        retL.append("\n-----------------WorkManagerWebApp().__dumpRequest()-----------------------------\n")
        retL.append("Parameter dictionary length = %d\n" % len(self.__myParameterDict))
        for k, vL in self.__myParameterDict.items():
            retL.append("Parameter %30s :" % k)
            for v in vL:
                retL.append(" ->  %s\n" % v)
        retL.append("-------------------------------------------------------------\n")
        return retL


class WorkManagerWebAppWorker(object):
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
         Worker methods for the chemical component editor application

         Performs URL - application mapping and application launching
         for chemical component editor tool.

         All operations can be driven from this interface which can
         supplied with control information from web application request
         or from a testing application.
        """
        self.__verbose = verbose
        self.__lfh = log
        self.__reqObj = reqObj
        self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
        self.__cI = ConfigInfo(self.__siteId)
        #
        self.__appPathD = {'/service/environment/dump': '_dumpOp',
                           '/service/workmanager/login': '_LoginOp',
                           '/service/workmanager/logout': '_LogoutOp',
                           '/service/workmanager/summary': '_SummaryOp',
                           '/service/workmanager/replacementhist': '_ReplacementHistOp',
                           '/service/workmanager/snapshotdiff': '_SnapshotDiffOp',
                           '/service/workmanager/getpassword': '_PasswordOp',
                           '/service/workmanager/milestonearchive': '_MilestoneOp',
                           '/service/workmanager/milestonereset': '_MilestoneResetOp',
                           '/service/workmanager/allowupload': '_AllowUploadOp',
                           '/service/workmanager/preventupload': '_PreventUploadOp',
                           '/service/workmanager/enableftpupload': '_EnableFtpUploadOp',
                           '/service/workmanager/allowsubmit': '_AllowSubmitOp',
                           '/service/workmanager/preventsubmit': '_PreventSubmitOp',
                           '/service/workmanager/rerunworkflow': '_RerunWorkFlowOp',
                           '/service/workmanager/killworkflow': '_KillWorkFlowOp',
                           '/service/workmanager/ciffile': '_DownloadCifFileOp',
                           '/service/workmanager/logfile': '_DownloadLogFileOp',
                           '/service/workmanager/filereports': '_ViewAllFileOp',
                           '/service/workmanager/gettabledata': '_GetTableDataOp',
                           '/service/workmanager/search': '_SearchPageOp',
                           '/service/workmanager/edit_my_list': '_EditMyListOp',
                           '/service/workmanager/refresh': '_RefreshOp',
                           '/service/workmanager/runengine': '_RunEngineOp',
                           '/service/workmanager/assign': '_AssignOp',
                           '/service/workmanager/level2': '_Level2PageOp',
                           '/service/workmanager/edituserdata': '_EditUserOp',
                           '/service/workmanager/saveuserdata': '_SaveUserOp',
                           '/service/workmanager/changeprivilege': '_ChangePrivilege',
                           '/service/workmanager/changeactiveuser': '_ChangeActiveUser',
                           '/service/workmanager/start_group_workflow': '_GroupEngineDepictOp',
                           '/service/workmanager/start_group_worktask': '_GroupTaskDepictOp',
                           '/service/workmanager/get_ligand_list': '_GetLigandListOp',
                           '/service/workmanager/run_group_engine': '_RunGroupEngineOp',
                           '/service/workmanager/run_group_tasks': '_RunGroupTasksOp',
                           '/service/workmanager/open_editing_page': '_LaunchEditingPageOp',
                           '/service/workmanager/submit_editing_form': '_ProcessEditingFormOp',
                           '/service/workmanager/check_editing_form_status': '_CheckEditingFormStatusOp',
                           }

    def doOp(self):
        """Map operation to path and invoke operation.

            :Returns:

            Operation output is packaged in a ResponseContent() object.
        """
        return self.__doOpException()

    # def __doOpNoException(self):
    #     """Map operation to path and invoke operation.  No exception handling is performed.

    #         :Returns:

    #         Operation output is packaged in a ResponseContent() object.
    #     """
    #     #
    #     reqPath = self.__reqObj.getRequestPath()
    #     if reqPath not in self.__appPathD:
    #         # bail out if operation is unknown -
    #         rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
    #         rC.setError(errMsg='Unknown operation')
    #         return rC
    #     else:
    #         mth = getattr(self, self.__appPathD[reqPath], None)
    #         rC = mth()
    #     return rC

    def __doOpException(self):
        """Map operation to path and invoke operation.  Exceptions are caught within this method.

            :Returns:

            Operation output is packaged in a ResponseContent() object.
        """
        #
        try:
            reqPath = self.__reqObj.getRequestPath()
            if reqPath not in self.__appPathD:
                # bail out if operation is unknown -
                rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
                rC.setError(errMsg='Unknown operation')
            else:
                mth = getattr(self, self.__appPathD[reqPath], None)
                rC = mth()
            return rC
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self.__lfh)
            rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
            rC.setError(errMsg='Operation failure')
            return rC

    ################################################################################################################
    # ------------------------------------------------------------------------------------------------------------
    #      Top-level REST methods
    # ------------------------------------------------------------------------------------------------------------
    #
    def _dumpOp(self):
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setHtmlList(self.__reqObj.dump(format='html'))
        return rC

    def _LoginOp(self):
        """ Launch first-level interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._LoginOp() Starting now\n")
        #
        self.__getSession()
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        username = str(self.__reqObj.getValue('username'))
        password = str(self.__reqObj.getValue('password'))
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        userInfo = db.Autenticate(username, password)
        if userInfo:
            readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='level1_config.cif', verbose=self.__verbose, log=self.__lfh)
            configDict = readUtil.read()
            depictUtil = DepictLevel1(reqObj=self.__reqObj, statusDB=db, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
            rC.setHtmlText(depictUtil.getPageText(page_id='level1_tmplt'))
        else:
            myD = {}
            myD['sessionid'] = self.__sessionId
            myD['message'] = 'Invalid Login'
            rC.setHtmlText(self.__processTemplate('login_tmplt.html', myD))
        #
        return rC

    def _LogoutOp(self):
        """ Logout first-level interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._LogoutOp() logout\n")
        #
        self.__getSession()
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        myD = {}
        myD['sessionid'] = ''
        myD['message'] = ''
        rC.setHtmlText(self.__processTemplate('login_tmplt.html', myD))
        #
        return rC

    def _SummaryOp(self):
        """ Summary page interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._SummaryOp() Starting now\n")
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='summary_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictOther(reqObj=self.__reqObj, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
        depictUtil.SummaryPage()
        rC.setHtmlText(depictUtil.getPageText(page_id='summary_tmplt'))
        #
        return rC

    def _ReplacementHistOp(self):
        """ Author initiated replacement history page interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._ReplacementHistOp() Starting now\n")
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='replacement_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictOther(reqObj=self.__reqObj, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
        depictUtil.ReplacementPage()
        rC.setHtmlText(depictUtil.getPageText(page_id='replacement_tmplt'))
        #
        return rC

    def _SnapshotDiffOp(self):
        """ Summary page interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._SnapshotDiffOp() Starting now\n")
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        snapshot = DepictSnapShot(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setHtmlText(snapshot.getPageText())
        #
        return rC

    def _PasswordOp(self):
        """ Get Entry password
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._PasswordOp() Starting now\n")
        #
        self.__getSession()
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        deppw, error = self.__getPassword(depositionid)
        return self.__returnJsonObject(deppw, error)

    def __getPassword(self, depositionid):
        """ Get password for entry depositionid
        """
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        depInfo = db.getDepInfo(depositionid=depositionid)
        #
        deppw = ''
        if depInfo and ('deppw' in depInfo) and depInfo['deppw']:
            deppw = str(depInfo['deppw'])
        #
        error = ''
        if not deppw:
            error = "Can't find password for entry " + depositionid
        #
        return deppw, error

    def _MilestoneOp(self):
        """ Copy model milestone file
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._MilestoneOp() Starting now\n")
        #
        self.__getSession()
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        msf = MileStoneFile(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        error = ''
        if not msf.getInputFile(depositionid, 'archive', 'model', 'pdbx', 'latest'):
            error = "Can't find model file for entry " + depositionid + ' in archive directory.'
        elif not msf.getOutputFile(depositionid, 'archive', 'model-annotate', 'pdbx', 'next'):
            error = 'Copy model-annotate file failed for entry ' + depositionid + '.'
        #
        return self.__returnJsonObject('Successfully wrote milestone file', error)

    def _MilestoneResetOp(self):
        """ Reset model milestone file
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._MilestoneResetOp() Starting now\n")
        #
        self.__getSession()
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        msf = MileStoneFile(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        lastInst = db.getLastInstance(depositionid=depositionid)
        if lastInst and ('dep_exp_method' in lastInst) and str(lastInst['dep_exp_method']).upper() in \
           ('ELECTRON CRYSTALLOGRAPHY', 'CRYO-ELECTRON MICROSCOPY', 'ELECTRON TOMOGRAPHY', 'ELECTRON MICROSCOPY'):
            msf.setEmEntry()
        #
        error = ''
        if not msf.getInputFile(depositionid, 'archive', 'model', 'pdbx', 'latest'):
            error = "Can't find model file for entry " + depositionid + ' in archive directory.'
        elif not msf.getOutputFile(depositionid, 'deposit', 'model', 'pdbx', 'next'):
            error = 'Copy model file to deposit failed for entry ' + depositionid + '.'
        #
        return self.__returnJsonObject('OK', error)

    def _AllowUploadOp(self):
        """ Allow file to be uploaded in DepUI
        """
        return self.__allow_or_prevent_op('allow', '_AllowUploadOp()', 'uploadOK.pkl',
                                          'You have set this deposition to allow bad uploads !',
                                          'Failed to allow incomplete upload !')

    def _PreventUploadOp(self):
        """ Prevent file to be uploaded in DepUI
        """
        return self.__allow_or_prevent_op('prevent', '_PreventUploadOp()', 'uploadOK.pkl',
                                          'You have prevented bad uploads !', 'Failed to block incomplete upload !')

    def _AllowSubmitOp(self):
        """ Allow entry to be submit in DepUI
        """
        return self.__allow_or_prevent_op('allow', '_AllowSubmitOp()', 'submitOK.pkl',
                                          'You have set this deposition to allow incomplete Deposition !',
                                          'Failed to allow incomplete submission !')

    def _PreventSubmitOp(self):
        """ Prevent entry to be submit in DepUI
        """
        return self.__allow_or_prevent_op('prevent', '_PreventSubmitOp()', 'submitOK.pkl',
                                          'You have prevented incomplete Deposition !', 'Failed to block incomplete submission !')

    def _EnableFtpUploadOp(self):
        """ Allow ftp upload file
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._EnableFtpUploadOp() Starting now\n")
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        userFile = self.__cI.get('FTP_USER_FILE')
        userDbFile = self.__cI.get('FTP_DB_FILE')
        ftpPath = self.__cI.get('FTP_STORAGE_PATH')
        error = ''
        try:
            ftpDir = os.path.join(ftpPath, depositionid)
            if not os.path.exists(ftpDir):
                cmd = ['mkdir', '-p', ftpDir]
                subprocess.check_call(cmd)
            #
        except Exception as e:
            self.__lfh.write("+WorkManagerWebAppWorker._EnableFtpUploadOp() Failed to create FTP user folder %s\n" % str(e))
            error = 'Failed to create FTP user folder'
        #
        try:
            with open(userFile, 'r') as infile:
                data = infile.read()
                if depositionid not in data:
                    depPW, _err = self.__getPassword(depositionid)
                    if depPW:
                        outfile = open(userFile, 'a')
                        outfile.write(depositionid + '\n' + depPW + '\n')
                        outfile.close()
                        cmd = ['db_load', '-T', '-t', 'hash', '-f', userFile, userDbFile]
                        subprocess.check_call(cmd)
                    else:
                        if error:
                            error += '; '
                        #
                        error += "Can't find password for entry " + depositionid
                    #
                #
            #
        except Exception as e:
            self.__lfh.write("+WorkManagerWebAppWorker._EnableFtpUploadOp() Failed to add FTP user %s : %s\n" % (depositionid, str(e)))
            if error:
                error += '; '
            #
            error += 'Failed to add FTP user'
        #
        if error:
            return self.__returnJsonObject('', error)
        #
        text = ''
        try:
            # create a pickle file, which is used to check if FTP upload was allowed
            storage_pickle_path = self.__get_deposit_storage_pickle_path('externalUpload.pkl')
            self.__dump_deposit_storage_pickle(storage_pickle_path, 'File upload in depUI failed; FTP upload enabled by Annotator from WFM')
            # send instructions email to depositor
            subject = 'Instructions for wwPDB FTP upload for deposition ' + depositionid
            myD = {}
            myD['depositionid'] = depositionid
            myD['ftp_host_name'] = self.__cI.get('FTP_HOST_NAME')
            myD['ftp_port_number'] = self.__cI.get('FTP_PORT_NUMBER')
            myD['ftp_connect_details_0'] = self.__cI.get('FTP_CONNECT_DETAILS')[0]
            myD['ftp_connect_details_1'] = self.__cI.get('FTP_CONNECT_DETAILS')[1]
            myD['site_dep_email_url'] = self.__cI.get('SITE_DEP_EMAIL_URL')
            message = self.__processTemplate('ftp_message.txt', myD)
            am = AutoMessage(siteId=self.__siteId)
            ret = am.sendSingleMessage(depositionid, subject, message, p_tmpltType="instruction-ftp")  # CS 2024-04-04 add template type
            text = 'FTP upload enabled; email sent to the user'
            self.__lfh.write("+WorkManagerWebAppWorker._EnableFtpUploadOp() sent message status=%s\n" % ret)
        except Exception as e:
            self.__lfh.write("+WorkManagerWebAppWorker._EnableFtpUploadOp() Failed to enable file import and/or send instructions to the depositor %s\n" % str(e))
            error = 'Failed in enable file import to depUI; instructions not sent'
        #
        return self.__returnJsonObject(text, error)

    def _RerunWorkFlowOp(self):
        """ Re-run work flow
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._RerunWorkFlowOp() Starting now\n")
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        status = reRunWorkflow(depositionid)
        text = ''
        error = ''
        if status == 'OK':
            text = 'You have restart the Workflow !'
        else:
            error = 'Rerun was not set ' + status
        #
        return self.__returnJsonObject(text, error)

    def _KillWorkFlowOp(self):
        """ Kill work flow
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._KillWorkFlowOp() Starting now\n")
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        status = db.killWorkFlow(depositionid=depositionid)
        error = ''
        if not status == 'OK':
            error = status
        #
        return self.__returnJsonObject(status, error)

    def _DownloadCifFileOp(self):
        """ Download model cif file
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._DownloadCifFileOp() Starting now\n")
        #
        self.__getSession()
        #
        depositionid = str(self.__reqObj.getValue("identifier"))
        filesource = str(self.__reqObj.getValue("filesource"))
        version = str(self.__reqObj.getValue("version"))
        instance = str(self.__reqObj.getValue("instance"))
        #
        pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        filePath = pI.getFilePath(dataSetId=depositionid, wfInstanceId=instance, contentType='model', formatType='pdbx',
                                  fileSource=filesource, versionId=version, partNumber='1')
        #
        self.__reqObj.setReturnFormat(return_format="binary")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setBinaryFile(filePath, attachmentFlag=True, serveCompressed=False)
        return rC

    def _DownloadLogFileOp(self):
        """ Download latest version model cif file
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._DownloadLogFileOp() Starting now\n")
        #
        logUtil = LogFileUtil(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        filePath = logUtil.getLogFile()
        #
        self.__reqObj.setReturnFormat(return_format="binary")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setBinaryFile(filePath, attachmentFlag=True, serveCompressed=False)
        return rC

    def _ViewAllFileOp(self):
        """ View all files page interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._ViewAllFileOp() Starting now\n")
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='allfile_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictOther(reqObj=self.__reqObj, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
        depictUtil.AllFilePage()
        rC.setHtmlText(depictUtil.getPageText(page_id='allfile_tmplt'))
        #
        return rC

    def _GetTableDataOp(self):
        """ return table data content
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._GetTableDataOp() Starting now\n")
        #
        self.__getSession()
        #
        data = loadPickleFile(self.__sessionPath, self.__reqObj.getValue("picklefile"))
        if not data:
            data = []
        #
        rtrnDict = {}
        rtrnDict['table_rows'] = data
        return self.__returnJsonDict(rtrnDict)

    def _SearchPageOp(self):
        """ Search Page interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._SearchPageOp() Starting now\n")
        #
        sUtil = SearchUtil(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        sUtil.updateSql()
        return self.__refreshTableContent(None, self.__reqObj.getValue('index'), False)

    def _EditMyListOp(self):
        """ Add to/Remove from my list interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._EditMyListOp() Starting now\n")
        #
        depositionid = self.__reqObj.getValue("identifier")
        annotator = self.__reqObj.getValue("annotator")
        type = self.__reqObj.getValue("type")  # pylint: disable=redefined-builtin
        sdb = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        if type == 'add':
            sdb.addToMyList(depositionid, annotator)
        else:
            sdb.removeFromMyList(depositionid)
        #
        return self.__refreshTableContent(sdb, 'all', False)

    def _RefreshOp(self):
        """ Refresh list interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._RefreshOp() Starting now\n")
        #
        return self.__refreshTableContent(None, self.__reqObj.getValue('index'), True)

    def _RunEngineOp(self):
        """ Run work flow engine interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._RunEngineOp() Starting now\n")
        #
        type = str(self.__reqObj.getValue("type"))  # pylint: disable=redefined-builtin
        depositionid = str(self.__reqObj.getValue("identifier"))
        instanceid = str(self.__reqObj.getValue("instance"))
        classid = str(self.__reqObj.getValue("classID"))
        command = str(self.__reqObj.getValue("command"))
        version = str(self.__reqObj.getValue("version"))
        # method = str(self.__reqObj.getValue("method"))
        #
        sdb = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        if type == 'kill':
            classInfo = sdb.getWfClassByID(classID='Annotate')
        else:
            classInfo = sdb.getWfClassByID(classID=classid)
        if not classInfo:
            return self.__returnJsonObject('', "Start '" + classid + "' workflow failed.")
        #
        classFileName = classInfo['class_file']
        if type != 'rerun_single_module':
            if classFileName.find(".bf.xml") < 0:
                classFileName = 'Annotation.bf.xml:' + str(classid)
            #
        #
        dep_status = sdb.getDepositionStatus(depositionid=depositionid, instid=instanceid, classid=classid)
        com_status = sdb.getCommandStatus(depositionid=depositionid)
        startWF = False
        if type == 'init':
            if dep_status == 'INIT' and com_status == 'INIT':
                startWF = True
            #
        elif type == 'open':
            if dep_status == 'OPEN' and com_status == 'WORKING':
                startWF = True
            #
        elif type == 'restart':
            if com_status != 'INIT':
                msf = MileStoneFile(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
                msf.getInputFile(depositionid, 'archive', 'model', 'pdbx', 'original')
                msf.getOutputFile(depositionid, 'archive', 'model', 'pdbx', 'next')
                startWF = True
            #
        else:
            startWF = True
        #
        if not startWF:
            return self.__returnJsonObject('OK', '')
        #
        time.sleep(0.5)
        msg = sdb.insertCommunicationCommand(depositionid=depositionid, instid=instanceid, classid=classid, command=command,
                                             classname=classFileName, dataversion=version)
        if msg != 'OK':
            return self.__returnJsonObject('', msg)
        else:
            return self.__returnJsonObject(msg, '')
        #

    def _AssignOp(self):
        """ Assign interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._AssignOp() Starting now\n")
        #
        assign_pair_list = []
        assigned_data = str(self.__reqObj.getValue("assigned_data")).replace(' ', '').strip()
        slist = assigned_data.split(',')
        for pair in slist:
            list1 = pair.split(':')
            assign_pair_list.append(list1)
        #
        assignUtil = AnnotAssignUtil(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        assignUtil.updateAnnotatorAssignment(assignList=assign_pair_list)
        #
        sdb = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        sdb.updateAnnotatorAssignment(assignList=assign_pair_list)
        #
        return self.__refreshTableContent(sdb, str(self.__reqObj.getValue("tab_id")) + "_table_1", False)

    def _Level2PageOp(self):
        """ Level2 page interface
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._Level2PageOp() Starting now\n")
        #
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='level_2_3_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictOther(reqObj=self.__reqObj, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        level = str(self.__reqObj.getValue("level"))
        if level == 'level3':
            depictUtil.getLevelPageSetting('level3_tmplt')
            rC.setHtmlText(depictUtil.getPageText(page_id='level3_tmplt'))
        else:
            depictUtil.getLevelPageSetting('level2_tmplt')
            rC.setHtmlText(depictUtil.getPageText(page_id='level2_tmplt'))
        #
        return rC

    def _EditUserOp(self):
        """ Edit User Information
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._EditUserOp() Starting now\n")
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        user = str(self.__reqObj.getValue('user'))
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        userInfo = db.getUserByName(username=user)
        if userInfo:
            userInfo['sessionid'] = str(self.__reqObj.getValue('sessionid'))
            rC.setHtmlText(self.__processTemplate('edit_user_tmplt.html', userInfo))
        else:
            myD = {}
            myD['message'] = 'Invalid user name: ' + user
            rC.setHtmlText(self.__processTemplate('exit_user_tmplt.html', myD))
        #
        return rC

    def _SaveUserOp(self):
        """ Save User Information
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._SaveUserOp() Starting now\n")
        #
        successful_msg = ''
        failed_msg = ''
        user_name = self.__reqObj.getValue('user_name')
        data = {}
        for item in ('password', 'email', 'initials', 'first_name', 'last_name', 'da_group_id'):
            val = self.__reqObj.getValue(item)
            if val:
                data[item] = val
            #
        #
        if user_name and data:
            db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
            ret = db.runUpdate(table='da_users', where={'user_name': user_name}, data=data)
            if not ret or ret != 'OK':
                failed_msg = 'Update user information failed.'
            else:
                successful_msg = 'User Information Updated.'
            #
        else:
            failed_msg = 'Update user information failed.'
        #
        tab_id = self.__reqObj.getValue('tab_id')
        if tab_id:
            return self.__returnJsonObject(successful_msg, failed_msg)
        else:
            self.__reqObj.setReturnFormat(return_format="html")
            rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
            myD = {}
            if failed_msg:
                myD['message'] = failed_msg
            else:
                myD['message'] = successful_msg
            #
            rC.setHtmlText(self.__processTemplate('exit_user_tmplt.html', myD))
            return rC
        #

    def _ChangePrivilege(self):
        """ Change da_users.da_group_id
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._ChangePrivilege() Starting now\n")
        #
        user_name_list = self.__reqObj.getValue('user_names').split(',')
        if not user_name_list:
            return self.__returnJsonObject('', 'No user information provided.')
        #
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        if db:
            UserInfo = db.getUserByInitial(initial=self.__reqObj.getValue('annotator'))
            if UserInfo and ('site' in UserInfo):
                GroupInfo = db.getSiteGroupWithCode(site=UserInfo['site'], code=self.__reqObj.getValue('code'))
                if GroupInfo and ('group_id' in GroupInfo):
                    for user_name in user_name_list:
                        _ret = db.runUpdate(table='da_users', where={'user_name': user_name}, data={'da_group_id': str(GroupInfo['group_id'])})  # noqa: F841
                    #
                else:
                    return self.__returnJsonObject('', 'Change privilege failed.')
                #
            else:
                return self.__returnJsonObject('', 'Change privilege failed.')
            #
        else:
            return self.__returnJsonObject('', 'Change privilege failed.')
        #
        return self.__returnUpdatePage(db)

    def _ChangeActiveUser(self):
        """ Change da_users.active
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._ChangeActiveUser() Starting now\n")
        #
        user_name_list = self.__reqObj.getValue('user_names').split(',')
        if not user_name_list:
            return self.__returnJsonObject('', 'No user information provided.')
        #
        db = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        for user_name in user_name_list:
            active = self.__reqObj.getValue(user_name)
            if not active:
                continue
            #
            _ret = db.runUpdate(table='da_users', where={'user_name': user_name}, data={'active': active})  # noqa: F841
        #
        return self.__returnUpdatePage(db)

    def _GroupEngineDepictOp(self):
        """ Depict Start Workflow Engine Page for entries under group deposition
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._GroupEngineDepictOp() Starting now\n")
        #
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='group_level_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictGroup(reqObj=self.__reqObj, conFigObj=configDict, workFlow=True, verbose=self.__verbose, log=self.__lfh)
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setHtmlText(depictUtil.getPageText(page_id='group_workflow_tmplt'))
        return rC

    def _GroupTaskDepictOp(self):
        """ Depict Start Workflow Engine Page for entries under group deposition
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._GroupTaskDepictOp() Starting now\n")
        #
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='group_level_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictGroup(reqObj=self.__reqObj, conFigObj=configDict, workFlow=False, verbose=self.__verbose, log=self.__lfh)
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setHtmlText(depictUtil.getPageText(page_id='group_worktask_tmplt'))
        return rC

    def _GetLigandListOp(self):
        """ Get all ligand IDs associated with the deposited entries in a group
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._GetLigandListOp() Starting now\n")
        #
        ligFinder = LigandFinder(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        returnMap, errMsg = ligFinder.getLigandInfo()
        #
        rtrnDict = {}
        if returnMap:
            rtrnDict['map'] = returnMap
        else:
            rtrnDict['errorflag'] = True
            rtrnDict['errortext'] = errMsg
        #
        return self.__returnJsonDict(rtrnDict)

    def _RunGroupEngineOp(self):
        """ Run Workflow Engine for selected entries under group deposition
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._RunGroupEngineOp() Starting now\n")
        #
        successful_msg = ''
        failed_msg = ''
        class_comb = self.__reqObj.getValue('classID')
        if not class_comb:
            failed_msg = 'No workflow selected'
        #
        entryList = self.__reqObj.getValueList('entry_id')
        if not entryList:
            if failed_msg:
                failed_msg += '\n'
            #
            failed_msg += 'No entry selected.'
        #
        if failed_msg:
            return self.__returnJsonObject(successful_msg, failed_msg)
        #
        c_list = class_comb.split('_')
        command = c_list[0]
        classid = c_list[1]
        if c_list[2] == 'UI':
            classid = c_list[1] + c_list[2]
        #
        version = ''
        if command == 'restartGoWF':
            version = 'latest'
        #
        sdb = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        classInfo = sdb.getWfClassByID(classID=classid)
        if not classInfo:
            return self.__returnJsonObject('', "Start '" + classid + "' workflow failed.")
        #
        classFileName = classInfo['class_file']
        if c_list[2] == 'WF':
            if (classFileName != 'Annotation.bf.xml') and (classFileName != 'Annotation.xml') and (classFileName != 'wf_op_annot-main_fs_archive.xml'):
                classFileName = 'Annotation.bf.xml:' + str(classid)
            #
        #
        for entry_id in entryList:
            if successful_msg:
                successful_msg += '\n'
            #
            instanceid = ''
            lastWFInstance = sdb.getLastWFInstance(depositionid=entry_id, classid=classid)
            if lastWFInstance and ('wf_inst_id' in lastWFInstance) and lastWFInstance['wf_inst_id']:
                instanceid = lastWFInstance['wf_inst_id']
            #
            msg = sdb.insertCommunicationCommand(depositionid=entry_id, instid=instanceid, classid=classid,
                                                 command=command, classname=classFileName, dataversion=version)
            if msg != 'OK':
                successful_msg += msg
            else:
                successful_msg += "Start '" + classid + "' workflow for '" + entry_id + "'."
            #
        #
        return self.__returnJsonObject(successful_msg, failed_msg)

    def _RunGroupTasksOp(self):
        """ Run tasks for selected entries under group deposition
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._RunGroupTasksOp() Starting now\n")
        #
        successful_msg = ''
        failed_msg = ''
        entryList = self.__reqObj.getValueList('entry_id')
        if not entryList:
            if failed_msg:
                failed_msg += '\n'
            #
            failed_msg += 'No entry selected.'
        #
        if failed_msg:
            return self.__returnJsonObject(successful_msg, failed_msg)
        #
        option = self.__reqObj.getValue('option')
        if (option == 'cifcheck') or (option == 'mischeck'):
            checkUtil = CifChecker(reqObj=self.__reqObj, entryList=entryList, verbose=self.__verbose, log=self.__lfh)
            successful_msg = checkUtil.run()
        elif option == 'copy_to_deposition':
            copyUtil = CopyFileToAutoGroup(reqObj=self.__reqObj, entryList=entryList, verbose=self.__verbose, log=self.__lfh)
            successful_msg = copyUtil.run()
        elif option == 'database':
            dbLoader = DBLoader(reqObj=self.__reqObj, entryList=entryList, verbose=self.__verbose, log=self.__lfh)
            successful_msg = dbLoader.run()
        elif option == 'ligand':
            return self.__returnJsonObject('"Merge Ligand Assignment" has not been implemented yet!', '')
        elif option == 'pdbfile':
            pfGenUtil = PdbFileGenerator(reqObj=self.__reqObj, entryList=entryList, verbose=self.__verbose, log=self.__lfh)
            successful_msg = pfGenUtil.run()
        elif option == 'sequence':
            self.__getSession()
            #
            filePath, errMsg = self.__getTemplateFile()
            if errMsg:
                return self.__returnJsonObject('', errMsg)
            elif not filePath:
                return self.__returnJsonObject('', 'No template model file found.')
            elif not os.access(filePath, os.F_OK):
                return self.__returnJsonObject('', "Template file '" + filePath + "' can not be found.")
            #
            seqMerger = SequenceMerger(reqObj=self.__reqObj, entryList=entryList, templateFile=filePath, verbose=self.__verbose, log=self.__lfh)
            successful_msg = seqMerger.run()
        elif option == 'status':
            stUpdater = StatusUpdater(reqObj=self.__reqObj, entryList=entryList, verbose=self.__verbose, log=self.__lfh)
            successful_msg = stUpdater.run()
        elif option == 'other':
            checkList = self.__reqObj.getValueList('checked_list')
            if len(checkList) == 0:
                return self.__returnJsonObject('', 'No task selected.')
            #
            needTemplateFile = False
            if (len(checkList) > 1) or (checkList[0] != 'loi'):
                needTemplateFile = True
            #
            self.__getSession()
            #
            filePath = ''
            if needTemplateFile:
                filePath, errMsg = self.__getTemplateFile()
                if errMsg:
                    return self.__returnJsonObject('', errMsg)
                elif not filePath:
                    return self.__returnJsonObject('', 'No template model file found.')
                elif not os.access(filePath, os.F_OK):
                    return self.__returnJsonObject('', "Template file '" + filePath + "' can not be found.")
                #
            #
            metaMerger = MetaDataMerger(reqObj=self.__reqObj, entryList=entryList, taskList=checkList, templateFile=filePath,
                                        verbose=self.__verbose, log=self.__lfh)
            successful_msg = metaMerger.run()
        elif option == 'recover':
            checkList = self.__reqObj.getValueList('checked_list')
            if len(checkList) == 0:
                return self.__returnJsonObject('', 'No task selected.')
            #
            metaMerger = MetaDataMerger(reqObj=self.__reqObj, entryList=entryList, taskList=checkList, recoverFlag=True,
                                        verbose=self.__verbose, log=self.__lfh)
            successful_msg = metaMerger.run()
        else:
            return self.__returnJsonObject('', 'No task was defined!')
        #
        return self.__returnJsonObject(successful_msg, failed_msg)

    def _LaunchEditingPageOp(self):
        """ Launch batch editing page
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._LaunchEditingPageOp() Starting now\n")
        #
        self.__getSession()
        #
        self.__reqObj.setReturnFormat(return_format="html")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        myD = {}
        myD['sessionid'] = self.__sessionId
        myD['annotator'] = str(self.__reqObj.getValue('annotator'))
        rC.setHtmlText(self.__processTemplate('editing_tmplt.html', myD))
        #
        return rC

    def _ProcessEditingFormOp(self):
        """ Process batch editing page
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._ProcessEditingFormOp() Starting now\n")
        #
        self.__getSession()
        #
        dU = DetachUtils(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        mdEditor = MetaDataEditor(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        dU.set(workerObj=mdEditor, workerMethod="ProcessForm")
        dU.runDetach()
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.setStatusCode('running')
        return rC

    def _CheckEditingFormStatusOp(self):
        """ Checking batch editing processing status
        """
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker._CheckEditingFormStatusOp() Starting now\n")
        #
        self.__getSession()
        #
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        #
        sph = self.__reqObj.getSemaphore()
        dU = DetachUtils(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        if (dU.semaphoreExists(sph)):
            mdEditor = MetaDataEditor(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
            rC.addDictionaryItems(mdEditor.GetFormProcessingResult())
        else:
            time.sleep(2)
            rC.setStatusCode('running')
        #
        return rC

    def __returnUpdatePage(self, db):
        """
        """
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='level1_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depictUtil = DepictBase(reqObj=self.__reqObj, statusDB=db, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
        _return_page = depictUtil.getPageText(page_id=self.__reqObj.getValue('return_page'))  # noqa: F841
        return self.__returnJsonObject(depictUtil.getPageText(page_id=self.__reqObj.getValue('return_page')), '')

    def __returnJsonObject(self, successful_msg, failed_msg):
        """ return json object
        """
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        if failed_msg:
            rC.setError(errMsg=failed_msg)
        else:
            rC.setText(text=successful_msg)
        #
        return rC

    def __returnJsonDict(self, rtrnDict):
        self.__reqObj.setReturnFormat(return_format="json")
        rC = ResponseContent(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
        rC.addDictionaryItems(rtrnDict)
        return rC

    def __refreshTableContent(self, sdb, index, flag):
        """
        """
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='level1_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        depContUtil = DepictContent(reqObj=self.__reqObj, statusDB=sdb, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
        returnMap = depContUtil.depictTableContent(index)
        rtrnDict = {}
        if returnMap:
            rtrnDict['status'] = 'OK'
            rtrnDict['map'] = returnMap
            if (index == 'all') and flag:
                if sdb is None:
                    sdb = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
                #
                sInfoUtil = ServerInfoUtil(reqObj=self.__reqObj, statusDB=sdb, conFigObj=configDict, verbose=self.__verbose, log=self.__lfh)
                text = sInfoUtil.getServerInfo()
                if text:
                    rtrnDict['serverinfotext'] = text
                #
            #
        else:
            rtrnDict['status'] = 'Failed'
        #
        return self.__returnJsonDict(rtrnDict)

    def __allow_or_prevent_op(self, op, function, pickle_file, successful_msg, failed_msg):
        if (self.__verbose):
            self.__lfh.write("+WorkManagerWebAppWorker.%s Starting now\n" % function)
        #
        storage_pickle_path = self.__get_deposit_storage_pickle_path(pickle_file)
        text = ''
        error = ''
        try:
            if op == 'allow':
                self.__dump_deposit_storage_pickle(storage_pickle_path, 'Unlocked by Annotator from WFM')
            elif op == 'prevent':
                if os.path.exists(storage_pickle_path):
                    os.remove(storage_pickle_path)
                #
            #
            text = successful_msg
        except:  # noqa: E722 pylint: disable=bare-except
            error = failed_msg
        #
        return self.__returnJsonObject(text, error)

    def __get_deposit_storage_pickle_path(self, pickle_file):
        """
        """
        spath = getPicklePath(str(self.__reqObj.getValue("identifier")))
        if spath:
            return os.path.join(spath, pickle_file)
        else:
            return None

    def __dump_deposit_storage_pickle(self, storage_pickle_path, reason):
        """
        """
        data = {}
        data['annotator_initials'] = str(self.__reqObj.getValue("annotator"))
        data['reason'] = reason
        data['date'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        f = open(storage_pickle_path, 'wb')
        pickle.dump(data, f)
        f.close()

    def __getSession(self):
        """ Join existing session or create new session as required.
        """
        #
        # pylint: disable=attribute-defined-outside-init
        self.__sObj = self.__reqObj.newSessionObj()
        self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        # self.__rltvSessionPath = self.__sObj.getRelativePath()
        # pylint: enable=attribute-defined-outside-init
        if (self.__verbose):
            self.__lfh.write("------------------------------------------------------\n")
            self.__lfh.write("+WorkManagerWebAppWorker.__getSession() - creating/joining session %s\n" % self.__sessionId)
            self.__lfh.write("+WorkManagerWebAppWorker.__getSession() - session path %s\n" % self.__sessionPath)

    def __getTemplateFile(self):
        """ Get template file for ('sequence', 'other') options
        """
        filePath = ''
        template_identifier = str(self.__reqObj.getValue("template_identifier"))
        if template_identifier:
            version = str(self.__reqObj.getValue("template_file_version"))
            if not version:
                version = 'latest'
            #
            pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
            filePath = pI.getFilePath(dataSetId=template_identifier, wfInstanceId=None, contentType='model', formatType='pdbx',
                                      fileSource='archive', versionId=version, partNumber='1')
            #
            if not filePath:
                return '', 'No template file found for Deposition ID=' + template_identifier + ' with version number=' + version + '.'
            #
        else:
            wuu = WebUploadUtils(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
            if not wuu.isFileUpload(fileTag='template_file'):
                return '', 'No upload template model file found.'
            #
            uploadFileName = wuu.copyToSession(fileTag='template_file')
            if not os.access(os.path.join(self.__sessionPath, uploadFileName), os.F_OK):
                return '', "Template file '" + os.path.join(self.__sessionPath, uploadFileName) + "' can not be found."
            #
            filePath = os.path.join(self.__sessionPath, 'uploadTemplateFile.cif')
            os.rename(os.path.join(self.__sessionPath, uploadFileName), filePath)
        #
        return filePath, ''

    def __processTemplate(self, fn, parameterDict=None):
        """ Read the input HTML template data file and perform the key/value substitutions in the
            input parameter dictionary.

            :Params:
                ``parameterDict``: dictionary where
                key = name of subsitution placeholder in the template and
                value = data to be used to substitute information for the placeholder

            :Returns:
                string representing entirety of content with subsitution placeholders now replaced with data
        """
        if parameterDict is None:
            parameterDict = {}
        tPath = self.__reqObj.getValue("TemplatePath")
        fPath = os.path.join(tPath, fn)
        ifh = open(fPath, 'r')
        sIn = ifh.read()
        ifh.close()
        return (sIn % parameterDict)



##
# File:  DepictLevel1.py
# Date:  17-Mar-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys

from wwpdb.apps.workmanager.depict.DepictBase import DepictBase
from wwpdb.apps.workmanager.depict.Level1Util import Level1Util
from wwpdb.apps.workmanager.depict.ServerInfoUtil import ServerInfoUtil


class DepictLevel1(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(DepictLevel1, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        #
        self.__setup()

    def __setup(self):
        """
        """
        dataD = {}
        dataD['breadcrumbs'] = '[1]Deposition Summary'
        dataD['pageTitle'] = 'Deposition Summary : Level 1 :'
        dataD['comment_start'] = ' '
        dataD['comment_end'] = ' '
        #
        self._connectStatusDB()
        self._getUserInfoDict()
        #
        if self._userInfo and ('code' in self._userInfo) and self._userInfo['code'] == 'DEP':
            dataD['breadcrumbs'] = '[1]Summary of entries being deposited'
            dataD['comment_start'] = '<!--'
            dataD['comment_end'] = '-->'
        #
        if self._userInfo and ('code' in self._userInfo) and self._userInfo['code'] == 'LANN':
            dataD['pageTitle'] = '[Level 1] Lead Annotator Screen'
            dataD['comment_start'] = '<!--'
            dataD['comment_end'] = '-->'
        #
        self._dataInfo['level1_tmplt'] = [dataD]
        level1Util = Level1Util(reqObj=self._reqObj, statusDB=self._statusDB, conFigObj=self._conFigObj, verbose=self._verbose, log=self._lfh)
        self._UtilClass['Level1Util'] = level1Util
        sInfoUtil = ServerInfoUtil(reqObj=self._reqObj, statusDB=self._statusDB, conFigObj=self._conFigObj, verbose=self._verbose, log=self._lfh)
        self._UtilClass['ServerInfoUtil'] = sInfoUtil



##
# File:  StatusUpdater.py
# Date:  25-Apr-2017
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import sys
import traceback

from wwpdb.apps.workmanager.db_access.ContentDbApi import ContentDbApi
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass
from wwpdb.io.file.mmCIFUtil import mmCIFUtil
from wwpdb.utils.db.StatusHistoryUtils import StatusHistoryUtils
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
#


class StatusUpdater(BaseClass):
    def __init__(self, reqObj=None, entryList=None, verbose=False, log=sys.stderr):
        """
        """
        super(StatusUpdater, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        self.__entryList = entryList
        self.__statusTokens = ('status_code', 'author_approval_type', 'author_release_status_code', 'date_hold_coordinates', 'pdbx_annotator', 'process_site')
        self.__depositionTableMap = {'status_code': 'status_code', 'author_release_status_code': 'author_release_status_code',
                                     'pdbx_annotator': 'annotator_initials', 'process_site': 'process_site'}
        self.__rcsb_statusTableMap = {'status_code': 'status_code', 'author_approval_type': 'author_approval_type',
                                      'author_release_status_code': 'author_release_status_code', 'date_hold_coordinates': 'date_hold_coordinates',
                                      'pdbx_annotator': 'rcsb_annotator', 'process_site': 'process_site'}
        self.__statusInfo = {}
        self.__depositionInfo = {}
        self.__rcsb_statusInfo = {}
        self.__getStatusInfo()
        self.__writeStatusInfoCif()

    def run(self):
        """
        """
        if not os.access(os.path.join(self._sessionPath, 'statusInfo_StatusUpdater.cif'), os.F_OK):
            return 'No status info. selected.'
        #
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        self.__updateStatusHistory()
        return self.__getReturnMessage()

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runSingle(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __getStatusInfo(self):
        for token in self.__statusTokens:
            value = self._reqObj.getValue(token)
            if not value:
                continue
            #
            self.__statusInfo[token] = value
            if token in self.__depositionTableMap:
                self.__depositionInfo[self.__depositionTableMap[token]] = value
            #
            if token in self.__rcsb_statusTableMap:
                self.__rcsb_statusInfo[self.__rcsb_statusTableMap[token]] = value
            #
        #

    def __writeStatusInfoCif(self):
        self._removeFile(os.path.join(self._sessionPath, 'statusInfo_StatusUpdater.cif'))
        if not self.__statusInfo:
            return
        #
        cifUtil = mmCIFUtil()
        cifUtil.AddBlock("STATUS")
        cifUtil.AddCategory('pdbx_database_status', self.__statusTokens)
        for key, value in self.__statusInfo.items():
            cifUtil.UpdateSingleRowValue('pdbx_database_status', key, 0, value)
        #
        cifUtil.WriteCif(os.path.join(self._sessionPath, 'statusInfo_StatusUpdater.cif'))

    def __runSingle(self, entry_id):
        message, modelFile = self._getExistingArchiveFile(entry_id, 'model', 'pdbx', 'latest')
        if message:
            self._dumpPickle(entry_id + "_StatusUpdater", message)
            return
        #
        message, updatedModelFile = self.__updateModelFile(entry_id, modelFile)
        if message:
            self._dumpPickle(entry_id + "_StatusUpdater", message)
            return
        #
        archiveModelFile = self._findArchiveFileName(entry_id, 'model', 'pdbx', 'next')
        message = self._copyFileUtil(updatedModelFile, archiveModelFile)
        if message:
            self._dumpPickle(entry_id + "_StatusUpdater", message)
        else:
            self._dumpPickle(entry_id + "_StatusUpdater", 'OK')
        #

    def __updateModelFile(self, entry_id, inputFile):
        updatedModelFile = entry_id + "_StatusUpdater.cif"
        self._removeFile(os.path.join(self._sessionPath, updatedModelFile))
        logFile = 'StatusUpdater_update_cif_' + entry_id + '.log'
        clogFile = 'StatusUpdater_update_cif_command_' + entry_id + '.log'
        cmd = self._getCmd('${BINPATH}/UpdateCifCategory', inputFile, updatedModelFile, logFile, clogFile, ' -data statusInfo_StatusUpdater.cif ')
        self._runCmd(cmd)
        if os.access(os.path.join(self._sessionPath, updatedModelFile), os.F_OK):
            return '', os.path.join(self._sessionPath, updatedModelFile)
        #
        return "Status update failed.", ""

    def __updateStatusHistory(self):
        """
        """
        try:
            annotator = self._reqObj.getValue("annotator")
            status_code = self._reqObj.getValue("status_code")
            if (not annotator) or (not status_code):
                return
            #
            statusHUtils = StatusHistoryUtils(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
            #
            for entry_id in self.__entryList:
                statusHistoryFilePath = self._pI.getStatusHistoryFilePath(dataSetId=entry_id, fileSource="archive", versionId="latest")
                if os.access(statusHistoryFilePath, os.R_OK):
                    continue
                #
                statusHUtils.createHistory([entry_id])
            #
            okShLoad = False
            if statusHUtils.updateEntryStatusHistory(entryIdList=self.__entryList, statusCode=status_code, annotatorInitials=annotator,
                                                     details="Update by group status update"):
                okShLoad = statusHUtils.loadEntryStatusHistory(entryIdList=self.__entryList)
            #
            if (self._verbose):
                self._lfh.write("+StatusUpdater.__updateStatusHistory() %r status history database load status %r\n" % (self.__entryList, okShLoad))
            #
        except:  # noqa: E722 pylint: disable=bare-except
            if (self._verbose):
                self._lfh.write("+StatusUpdater.__updateStatusHistory() %s status history update and database load failed with exception\n")
                traceback.print_exc(file=self._lfh)
            #
        #

    def __getReturnMessage(self):
        message = ''
        updatedList = []
        for entry_id in self.__entryList:
            pickleData = self._loadPickle(entry_id + "_StatusUpdater")
            if pickleData and pickleData != 'OK':
                message += "Status update for " + entry_id + " failed:\n\t" + pickleData + "\n"
            else:
                message += "Status update for " + entry_id + " successfully.\n"
                updatedList.append(entry_id)
            #
            self._removePickle(entry_id + "_StatusUpdater")
        #
        if updatedList:
            if self.__depositionInfo:
                statusDB = StatusDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
                for entry_id in updatedList:
                    statusDB.runUpdate(table='deposition', where={'dep_set_id' : entry_id}, data=self.__depositionInfo)
                #
            #
            if self.__rcsb_statusInfo:
                contentDB = ContentDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
                for entry_id in updatedList:
                    contentDB.runUpdate(table='rcsb_status', where={'Structure_ID' : entry_id}, data=self.__rcsb_statusInfo)
                #
            #
        #
        return message


if __name__ == '__main__':
    from wwpdb.utils.session.WebRequest import InputRequest
    from wwpdb.utils.config.ConfigInfo import ConfigInfo
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002030")
    myReqObj.setValue("sessionid", "88626e0cc0b1a1bbd10bb2df8a0d68573fcbd5fe")
    myReqObj.setValue("status_code", "HPUB")
    myReqObj.setValue("author_release_status_code", "REL")
    myReqObj.setValue("date_hold_coordinates", "2017-04-25")
    myReqObj.setValue("pdbx_annotator", "LD")
    myReqObj.setValue("author_approval_type", "implicit")
    myentryList = ['D_8000210285', 'D_8000210286']
    pfGenUtil = StatusUpdater(reqObj=myReqObj, entryList=myentryList, verbose=False, log=sys.stderr)
    print(pfGenUtil.run())



##
# File:  DBLoader.py
# Date:  01-Jul-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCommon
from wwpdb.utils.db.DBLoadUtil import DBLoadUtil
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.io.file.mmCIFUtil import mmCIFUtil
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
from wwpdb.io.locator.PathInfo import PathInfo
#


class DBLoader(object):
    def __init__(self, reqObj=None, entryList=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__entryList = entryList
        self.__verbose = verbose
        self.__lfh = log
        self.__siteId = str(self.__reqObj.getValue('WWPDB_SITE_ID'))
        self.__cICommon = ConfigInfoAppCommon(self.__siteId)
        #
        self.__rcsbRoot = self.__cICommon.get_site_annot_tools_path()
        self.__compRoot = self.__cICommon.get_site_cc_cvs_path()
        #
        # self.__sessionId = None
        self.__sessionPath = None
        self.__getSession()
        self.__pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        self.__returnMessage = ''

    def run(self):
        """
        """
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self.__sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        self.__runDBLoading()
        return self.__returnMessage

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            sourceFile = self.__pI.getFilePath(dataSetId=entry_id, wfInstanceId=None, contentType='model', formatType='pdbx',
                                               fileSource='archive', versionId='latest', partNumber='1')
            if (not sourceFile) or (not os.access(sourceFile, os.F_OK)):
                continue
            #
            cmd = 'cd ' + self.__sessionPath + ' ; RCSBROOT=' + self.__rcsbRoot + ' ; export RCSBROOT ; COMP_PATH=' \
                + self.__compRoot + ' ; export COMP_PATH ; BINPATH=${RCSBROOT}/bin; export BINPATH; ${BINPATH}/GetDepositionInfo ' \
                + ' -input ' + sourceFile + ' -output ' + entry_id + '_db_info.cif -log ' + entry_id + '_db.log > ' + entry_id \
                + '_db_command.log  2>&1 ; '
            os.system(cmd)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __runDBLoading(self):
        """
        """
        statusDB = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        file_list = []
        for entry_id in self.__entryList:
            sourceFile = self.__pI.getFilePath(dataSetId=entry_id, wfInstanceId=None, contentType='model', formatType='pdbx',
                                               fileSource='archive', versionId='latest', partNumber='1')
            if sourceFile and os.access(sourceFile, os.F_OK):
                file_list.append(sourceFile)
            #
            info_file = os.path.join(self.__sessionPath, entry_id + '_db_info.cif')
            if os.access(info_file, os.F_OK):
                cifObj = mmCIFUtil(filePath=info_file)
                vList = cifObj.GetValue('info')
                if vList:
                    info_data = vList[0]
                    if info_data:
                        self.__returnMessage += 'Loaded ' + entry_id + ' successfully.\n'
                        statusDB.runUpdate(table='deposition', where={'dep_set_id' : entry_id}, data=info_data)
                    else:
                        self.__returnMessage += 'Loading ' + entry_id + ' failed.\n'
                    #
                else:
                    self.__returnMessage += 'Loading ' + entry_id + ' failed.\n'
                #
            else:
                self.__returnMessage += 'Loading ' + entry_id + ' failed.\n'
            #
        #
        if file_list:
            dbLoader = DBLoadUtil(reqObj=self.__reqObj, verbose=self.__verbose, log=self.__lfh)
            dbLoader.doLoading(file_list)
        #

    def __getSession(self):
        """
        """
        self.__sObj = self.__reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()


if __name__ == '__main__':
    from wwpdb.utils.session.WebRequest import InputRequest
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002003")
    myReqObj.setValue("sessionid", " b8030220bdf3559c10a2c63618cd85a25256f7c1")
    myentryList = ['D_8000200175', 'D_8000200176']
    copyUtil = DBLoader(reqObj=myReqObj, entryList=myentryList, verbose=False, log=sys.stderr)
    print(copyUtil.run())



##
# File:  MetaDataMerger.py
# Date:  30-Mar-2020
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2020 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import sys

from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
#


class MetaDataMerger(BaseClass):
    def __init__(self, reqObj=None, entryList=None, taskList=None, recoverFlag=False, templateFile=None, verbose=False, log=sys.stderr):
        """
        """
        super(MetaDataMerger, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        self.__entryList = entryList
        self.__taskList = taskList
        self.__recoverFlag = recoverFlag
        self.__templateFile = templateFile
        self.__loiMap = {}
        self.__loiMsg = ""

    def run(self):
        """
        """
        if "loi" in self.__taskList:
            self.__getLOIMap()
        #
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        return self.__getReturnMessage()

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runSingle(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __getLOIMap(self):
        """
        """
        for entry_id in self.__entryList:
            ligList = self._reqObj.getValueList("ligand_" + entry_id)
            if ligList:
                self.__loiMap[entry_id] = ligList
            #
        #
        if not self.__loiMap:
            self.__loiMsg = "No ligand selected for 'Ligand of interesting' task.\n"
        #

    def __runSingle(self, entry_id):
        """
        """
        message, modelFile = self._getExistingArchiveFile(entry_id, "model", "pdbx", "latest")
        if message:
            self._dumpPickle(entry_id + "_MetaDataMerger", message)
            return
        #
        if self.__recoverFlag:
            if "revision" in self.__taskList:
                message, templateFile = self._getExistingArchiveFile(entry_id, "model-release", "pdbx", "latest")
            else:
                message, templateFile = self._getExistingArchiveFile(entry_id, "model", "pdbx", "1")
            #
            if message:
                self._dumpPickle(entry_id + "_MetaDataMerger", message)
                return
            #
            message, updatedModelFile = self.__updateModelFile(entry_id, modelFile, templateFile)
        else:
            message, updatedModelFile = self.__updateModelFile(entry_id, modelFile, self.__templateFile)
        #
        if message:
            self._dumpPickle(entry_id + "_MetaDataMerger", message)
            return
        #
        archiveModelFile = self._findArchiveFileName(entry_id, "model", "pdbx", "next")
        message = self._copyFileUtil(updatedModelFile, archiveModelFile)
        if message:
            self._dumpPickle(entry_id + "_MetaDataMerger", message)
        else:
            self._dumpPickle(entry_id + "_MetaDataMerger", "OK")
        #

    def __updateModelFile(self, entry_id, inputFile, templateFile):
        """
        """
        updatedModelFile = entry_id + "_MetaDataMerger.cif"
        self._removeFile(os.path.join(self._sessionPath, updatedModelFile))
        logFile = "MetaDataMerger_update_cif_" + entry_id + ".log"
        self._removeFile(os.path.join(self._sessionPath, logFile))
        clogFile = "MetaDataMerger_update_cif_command_" + entry_id + ".log"
        self._removeFile(os.path.join(self._sessionPath, clogFile))
        #
        option = " -task " + ",".join(self.__taskList)
        if templateFile:
            option += " -example " + templateFile
        #
        if (entry_id in self.__loiMap) and self.__loiMap[entry_id]:
            option += " -ligand " + ",".join(self.__loiMap[entry_id])
        #
        cmd = self._getCmd("${BINPATH}/MergeMetaDataApp", inputFile, updatedModelFile, logFile, clogFile, option)
        self._runCmd(cmd)
        #
        msg = self._getLogMessage(os.path.join(self._sessionPath, logFile))
        cmsg = self._getLogMessage(os.path.join(self._sessionPath, clogFile))
        if cmsg:
            if msg:
                msg += "\n"
            #
            msg += cmsg
        #
        if msg:
            return msg, ""
        #
        if os.access(os.path.join(self._sessionPath, updatedModelFile), os.F_OK):
            return "", os.path.join(self._sessionPath, updatedModelFile)
        #
        return "Merge meta data failed.", ""

    def __getReturnMessage(self):
        message = self.__loiMsg
        for entry_id in self.__entryList:
            pickleData = self._loadPickle(entry_id + "_MetaDataMerger")
            if pickleData and pickleData != "OK":
                message += "Merge meta data for " + entry_id + " failed:\n\t" + pickleData + "\n"
            else:
                message += "Merge meta data for " + entry_id + " successfully.\n"
            #
            self._removePickle(entry_id + "_MetaDataMerger")
        #
        return message



##
# File:  DepictGroup.py
# Date:  27-Jun-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys

from wwpdb.apps.workmanager.depict.DepictBase import DepictBase
from wwpdb.apps.workmanager.workflow_access.WorkflowXMLLoader import WorkflowXMLLoader


class DepictGroup(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, workFlow=False, verbose=False, log=sys.stderr):
        """
        """
        super(DepictGroup, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        #
        self.__workFlowFlag = workFlow
        self.__wfTaskList = []
        self.__entryList = []
        self.__colorCodeMap = {'WF': 'groupTaskWF', 'Bucket': 'groupTaskBucket', 'UI': 'groupTaskUI'}
        self.__setup()
        self.__get_workflow_info()
        self.__get_entry_info()

    def __setup(self):
        """
        """
        self._connectStatusDB()
        self._getUserInfoDict()
        #
        if self.__workFlowFlag:
            self.__wfTaskList.append({'name' : 'run Annotation WF', 'classID' : 'runWF_Annotate_WF', 'groupclass' : self.__colorCodeMap['WF']})
            self.__wfTaskList.append({'name' : 'restart Annotation WF', 'classID' : 'restartGoWF_Annotate_WF', 'groupclass' : self.__colorCodeMap['WF']})
            self.__classInfo = self._statusDB.getWfClassByID(classID='Annotate')
            wfloader = WorkflowXMLLoader(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
            wfloader.loadWorkFlowXMLFile(self.__classInfo['class_file'])
            wfWorkFlowInfo = wfloader.getWorkFlowInfo()
            for wfType in ('WF', 'Bucket'):
                for wf in wfWorkFlowInfo.values():
                    if 'classID' not in wf:
                        continue
                    #
                    myD = {}
                    myD['name'] = 'restart ' + wf['name'] + ' ' + wfType
                    myD['classID'] = 'restartGoWF_' + wf['classID'] + '_' + wfType
                    myD['groupclass'] = self.__colorCodeMap[wfType]
                    self.__wfTaskList.append(myD)
                #
            #
        #
        entryIdList = self._statusDB.getEntryListForGroup(groupids=[self._reqObj.getValue("identifier")])
        if entryIdList:
            for entry in entryIdList:
                if self.__workFlowFlag:
                    for item in ('sessionid', 'annotator'):
                        entry[item] = self._reqObj.getValue(item)
                    #
                #
                self.__entryList.append(entry)
            #
        #

    def __get_workflow_info(self):
        """
        """
        if not self.__wfTaskList:
            return
        #
        MaxTask = 5
        module_tmplt = self._getPageTemplate('module_tmplt')
        count = 0
        workflow_info = '<tr>\n'
        for wfTask in self.__wfTaskList:
            if count == MaxTask:
                count = 0
                workflow_info += '</tr>\n<tr>\n'
            #
            workflow_info += module_tmplt % wfTask
            count += 1
        #
        workflow_info += '</tr>\n<tr>\n'
        for wfMod in ('TransMod', 'LigMod', 'SeqMod', 'AnnMod', 'ValMod'):
            myD = {}
            myD['name'] = 'restart ' + wfMod + ' UI'
            myD['classID'] = 'restartGoWF_' + wfMod + '_UI'
            myD['groupclass'] = self.__colorCodeMap['UI']
            workflow_info += module_tmplt % myD
        #
        workflow_info += '</tr>\n'
        self._dataInfo['workflow_info'] = workflow_info

    def __get_entry_info(self):
        """
        """
        if self.__workFlowFlag:
            MaxEntry = 6
            entry_tmplt = self._getPageTemplate('entry_tmplt')
        else:
            MaxEntry = 2
            entry_tmplt = self._getPageTemplate('task_entry_tmplt')
        #
        count = 0
        entry_info = '<tr>\n'
        for entry in self.__entryList:
            if count == MaxEntry:
                count = 0
                entry_info += '</tr>\n<tr>\n'
            #
            entry_info += entry_tmplt % entry
            count += 1
        #
        entry_info += '</tr>\n'
        self._dataInfo['entry_info'] = entry_info



##
# File:  StatsUtil.py
# Date:  17-July-2015
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import sys
from datetime import date, timedelta

from wwpdb.apps.workmanager.db_access.ContentDbApi import ContentDbApi
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.workflow_access.OrderedDict import OrderedDict


class StatsUtil(object):
    """
    """
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):
        """
        """
        self.__siteId = siteId
        self.__verbose = verbose
        self.__lfh = log
        #
        self.__statusDB = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        self.__contentDB = ContentDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        #
        self.__week_day = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
        self.__today = date.today()
        self.__annoList = self.__statusDB.getActiveAnnoList()

    def getWeeklyStatus(self):
        """
        """
        index, TableColumn, TableData = self.__getInitialTableDef()
        total_dir = {}
        #
        one_day = timedelta(1)
        start_date = self.__today - timedelta(6)
        for _i in range(7):
            plist = self.__contentDB.getDailyStatsList(date=start_date)
            TableData, total_dir = self.__getProcessCount(plist, TableData, total_dir, index)
            date_string = self.__week_day[start_date.weekday()] + ' (' + str(start_date)[5:].replace('-', '/') + ')'
            TableColumn.insert(index, str(index), {'label' : date_string, 'data-field': str(index)})
            start_date += one_day
            index += 1
        #
        TableData = self.__processTotalCount(TableData, total_dir, index)
        TableColumn.insert(index, str(index), {'label' : 'Total', 'data-field': str(index)})
        index += 1
        TableData = self.__processAverageCount(TableData, total_dir, index, 5.0)
        TableColumn.insert(index, str(index), {'label' : 'Average', 'data-field': str(index)})
        return TableColumn.values(), TableData.values()

    def getMonthlyStats(self):
        """
        """
        start_date = self.__today
        week_day = start_date.weekday()
        if week_day > 0:
            start_date -= timedelta(week_day)
        #
        dates_list = []
        for i in range(3):
            start = start_date - timedelta(21 - i * 7)
            end = start + timedelta(6)
            dates_list.append([start, end])
        #
        dates_list.append([start_date, self.__today])
        #
        index, TableColumn, TableData = self.__getInitialTableDef()
        total_dir = {}
        #
        for dates in dates_list:
            slist = self.__contentDB.getRangeStatsList(startdate=dates[0], enddate=dates[1])
            TableData, total_dir = self.__getProcessCount(slist, TableData, total_dir, index)
            date_string = self.__week_day[dates[0].weekday()] + '(' + str(dates[0])[5:].replace('-', '/') + ') - ' \
                + self.__week_day[dates[1].weekday()] + '(' + str(dates[1])[5:].replace('-', '/') + ')'
            TableColumn.insert(index, str(index), {'label' : date_string, 'data-field': str(index)})
            index += 1
        #
        TableData = self.__processTotalCount(TableData, total_dir, index)
        TableColumn.insert(index, str(index), {'label' : 'Total', 'data-field': str(index)})
        index += 1
        TableData = self.__processAverageCount(TableData, total_dir, index, 4.0)
        TableColumn.insert(index, str(index), {'label' : 'Average', 'data-field': str(index)})
        return TableColumn.values(), TableData.values()

    def getProcessStats(self):
        """
        """
        return_list = self.__contentDB.getInProcessStatsList()
        return_dir = {}
        for row in return_list:
            if row['status_code'] in return_dir:
                return_dir[row['status_code']].append(row['rcsb_annotator'])
            else:
                return_dir[row['status_code']] = [row['rcsb_annotator']]
            #
        #
        index, TableColumn, TableData = self.__getInitialTableDef()
        total_dir = {}
        #
        for status in ('WAIT', 'PROC', 'AUTH', 'POLC', 'REPL'):
            found_list = []
            if status in return_dir:
                found_list = return_dir[status]
            #
            TableData, total_dir = self.__getProcessCount(found_list, TableData, total_dir, index)
            TableColumn.insert(index, str(index), {'label' : status, 'data-field': str(index)})
            index += 1
        #
        TableData = self.__processTotalCount(TableData, total_dir, index)
        TableColumn.insert(index, str(index), {'label' : 'Total', 'data-field': str(index)})
        return TableColumn.values(), TableData.values()

    def __getInitialTableDef(self):
        """
        """
        index = 0
        TableColumn = OrderedDict()
        TableColumn.insert(index, str(index), {'label' : 'Annotator', 'data-field': str(index)})
        TableData = OrderedDict()
        count = 0
        for anno in self.__annoList:
            TableData.insert(count, anno['initials'], {str(index) : anno['initials']})
            count += 1
        #
        TableData.insert(count, 'total', {str(index) : 'total'})
        index += 1
        return index, TableColumn, TableData

    def __getProcessCount(self, found_list, TableData, total_dir, index):
        """
        """
        dir = {}  # pylint: disable=redefined-builtin
        for anno in self.__annoList:
            dir[anno['initials']] = 0
        #
        total = 0
        if found_list:
            for ai in found_list:
                if ai not in dir:
                    continue
                #
                dir[ai] += 1
                total += 1
            #
        #
        dir['total'] = total
        #
        for k, v in dir.items():
            TableData[k][str(index)] = str(v)
            if k in total_dir:
                total_dir[k] += v
            else:
                total_dir[k] = v
            #
        #
        return TableData, total_dir

    def __processTotalCount(self, TableData, total_dir, index):
        """
        """
        for k, v in total_dir.items():
            TableData[k][str(index)] = str(v)
        #
        return TableData

    def __processAverageCount(self, TableData, total_dir, index, denominator):
        """
        """
        for k, v in total_dir.items():
            TableData[k][str(index)] = '%.2f' % (v / denominator)
        #
        return TableData


if __name__ == '__main__':
    st = StatsUtil(siteId='WWPDB_DEPLOY_TEST_RU', verbose=True, log=sys.stderr)
    column, data = st.getWeeklyStatus()
    print(column)
    print(data)
    column1, data1 = st.getMonthlyStats()
    print(column1)
    print(data1)
    column2, data2 = st.getProcessStats()
    print(column2)
    print(data2)



##
# File:  Level1Util.py
# Date:  18-Mar-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import os
import sys
# from types import *

from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCommon
from wwpdb.io.file.mmCIFUtil import mmCIFUtil
from wwpdb.apps.workmanager.depict.DepictBase import DepictBase
from wwpdb.apps.workmanager.depict.DepictContent import DepictContent
from wwpdb.apps.workmanager.depict.ReadConFigFile import dumpPickleFile


class Level1Util(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(Level1Util, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        #
        self.__sObj = self._reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        self.__userID = self._reqObj.getValue('username').upper()
        self.__tab_def_id = ''
        self.__tab_count_id = ''
        self.__resultMap = {}
        # tableContentMap = { 'tab_count_id_table_id': { 'data-field': [fildlist], 'pkl': tableContentPickleFile,
        #                    'sql': sql, 'sql_selection': sql_selection, 'tab_count_id': 'tab_count_id',
        #                    'entry_count': { 'tab_count': [ items ], 'status_count': [ items ] } } }
        self.__tableContentMap = {}

    def run(self):
        """
        """
        tabList = self.__getTabList()
        if not tabList:
            return ''
        #
        self.__countMap = {}  # pylint: disable=attribute-defined-outside-init
        self._getUserInfoDict()
        #
        self._dataInfo['ann_selection'] = self._getAnnotatorSelection()
        count = 1
        all_tab_ids = []
        for tab in tabList:
            self.__tab_def_id = tab
            self.__tab_count_id = 'id_' + str(count)
            all_tab_ids.append(self.__tab_count_id)
            if 'firstTab' not in self.__resultMap:
                self.__resultMap['firstTab'] = self.__tab_count_id
            #
            self._dataInfo['tab_id'] = self.__tab_count_id
            self.__processTableDefinition()
            self.__processTabDefinition()
            count += 1
        #
        self.__resultMap['all_tab_ids'] = ','.join(all_tab_ids)
        #
        for _k, v in self.__tableContentMap.items():
            if not v['tab_count_id'] in self.__countMap:
                continue
            #
            v['entry_count'] = self.__countMap[v['tab_count_id']]
        #
        dumpPickleFile(self.__sessionPath, 'TableContentMap.pkl', self.__tableContentMap)
        depContUtil = DepictContent(reqObj=self._reqObj, statusDB=self._statusDB, conFigObj=self._conFigObj, verbose=self._verbose, log=self._lfh)
        self.__resultMap['entry_count_map'] = depContUtil.depictTableContent('all')
        #
        return ''

    def getResult(self, input_type='', return_type='string', delimiter=','):
        """
        """
        if delimiter == 'newline':
            delimiter = '\n'
        elif delimiter == 'empty':
            delimiter = ''
        #
        if input_type and (input_type in self.__resultMap):
            return self._processResult(self.__resultMap[input_type], return_type, delimiter)
        #
        if return_type == 'dict':
            return '{}'
        elif return_type == 'list':
            return '[]'
        else:
            return ''

    def getEnum(self, input_type=''):
        """
        """
        if not input_type:
            return '[]'
        #
        enum_list = getattr(self, "_get_%s_enum_list" % input_type)()
        #
        if enum_list:
            return self._processResult(enum_list, 'list', ',')
        #
        return '[]'

    def __getTabList(self):
        """
        """
        templateID = 'default_template'
        #
        if ('user_template_mapping' in self._conFigObj) and (self.__userID in self._conFigObj['user_template_mapping']):
            templateID = self._conFigObj['user_template_mapping'][self.__userID]
        #
        if ('level1_template_definition' in self._conFigObj) and (templateID in self._conFigObj['level1_template_definition']):
            # return self._conFigObj['level1_template_definition'][templateID]
            templateList = []
            for dirMap in self._conFigObj['level1_template_definition'][templateID]:
                if ('condition_type' not in dirMap) or ('condition_value' not in dirMap):
                    templateList.append(dirMap['tab_id'])
                elif dirMap['condition_type'] == 'statusDB':
                    self._connectStatusDB()
                    if self._statusDB.isTableValid(table=dirMap['condition_value']):
                        templateList.append(dirMap['tab_id'])
                    #
                #
            #
            return templateList
        #
        return []

    def __processTableDefinition(self):
        """
        """
        if ('table_definition' not in self._conFigObj) or (self.__tab_def_id not in self._conFigObj['table_definition']):
            return
        #
        dataList = []
        tableLoad = []
        table_keys = sorted(self._conFigObj['table_definition'][self.__tab_def_id].keys())
        for key in table_keys:
            tableDef = self._conFigObj['table_definition'][self.__tab_def_id][key]
            tableContentFile = 'table_content_' + str(len(self.__tableContentMap) + 1) + '.pkl'
            tableLoad.append([tableDef['load'], tableDef['table_id'], tableContentFile])
            dataD = {}
            dataD['tab_id'] = self.__tab_count_id
            dataD['table_id'] = tableDef['table_id']
            dataD['display'] = tableDef['display']
            if 'table_title' in tableDef:
                dataD['table_title'] = tableDef['table_title']
            #
            dataD['table_option'] = tableDef['option']
            #
            if 'column' in tableDef:
                columnList, dataField = self.__getColumnListandDataField(tableDef['table_id'], tableDef['column'])
                dataD['column_labels'] = '\n'.join(columnList)
                self.__processTableContents(tableDef, dataField, tableContentFile)
            elif 'binding_function' in tableDef:
                tableMap = {}
                tableMap['tab_count_id'] = self.__tab_count_id
                tableMap['binding_function'] = tableDef['binding_function']
                tableMap['pkl'] = tableContentFile
                #
                if 'binding_class' in tableDef:
                    tableMap['binding_class'] = tableDef['binding_class']
                    self._processBindingClass(tableDef['binding_class'])
                    classUtil = self._UtilClass[tableDef['binding_class']]
                    columnDef, _data = getattr(classUtil, '%s' % tableDef['binding_function'])()
                else:
                    columnDef, _data = getattr(self, '%s' % tableDef['binding_function'])()
                #
                self.__tableContentMap[self.__tab_count_id + '_' + tableDef['table_id']] = tableMap
                columnList, dataField = self.__getColumnListandDataField(tableDef['table_id'], columnDef)
                dataD['column_labels'] = '\n'.join(columnList)
            #
            dataList.append(dataD)
        #
        if dataList:
            self._dataInfo['data_table_tmplt'] = dataList
        #
        if tableLoad:
            if 'table_id_map' not in self.__resultMap:
                self.__resultMap['table_id_map'] = {}
            #
            self.__resultMap['table_id_map'][self.__tab_count_id] = tableLoad
        #

    def __getColumnListandDataField(self, table_id, columnDefList):
        """
        """
        columnList = []
        dataField = []
        for vDict in columnDefList:
            columnDef = '<th'
            dataField.append(vDict['data-field'])
            for item in ('data-field', 'data-sortable', 'data-visible', 'data-sorter', 'data-sort-name', 'data-cell-style'):
                if item == 'data-visible':
                    key = self.__userID + '-' + self.__tab_def_id + '-' + table_id + '-' + vDict['data-field']
                    all_key = 'all-' + self.__tab_def_id + '-' + table_id + '-' + vDict['data-field']
                    if ('user_tab_table_column_config' in self._conFigObj) and (key in self._conFigObj['user_tab_table_column_config']):
                        if self._conFigObj['user_tab_table_column_config'][key] == 'false':
                            columnDef += ' ' + item + '="false"'
                        #
                    elif ('user_tab_table_column_config' in self._conFigObj) and (all_key in self._conFigObj['user_tab_table_column_config']):
                        if self._conFigObj['user_tab_table_column_config'][all_key] == 'false':
                            columnDef += ' ' + item + '="false"'
                        #
                    elif item in vDict:
                        columnDef += ' ' + item + '="' + vDict[item] + '"'
                    #
                elif item in vDict:
                    columnDef += ' ' + item + '="' + vDict[item] + '"'
                #
            #
            columnDef += '>' + vDict['label'] + '</th>'
            columnList.append(columnDef)
        #
        return columnList, dataField

    def __processTableContents(self, tableDef, dataField, tableContentFile):
        """
        """
        tableMap = {}
        tableMap['tab_count_id'] = self.__tab_count_id
        tableMap['data-field'] = dataField
        tableMap['pkl'] = tableContentFile
        for extraInfo in ('order_condition', 'sort_function'):
            if (extraInfo in tableDef) and tableDef[extraInfo]:
                tableMap[extraInfo] = tableDef[extraInfo]
            #
        #
        sql_selection = tableDef['sql_selection']
        if 'sql_where_condition' in tableDef:
            # sql_where_condition = tableDef['sql_where_condition']
            if 'sql_variable' in tableDef:
                if tableDef['sql_variable'] == 'retired_annotator':
                    myD = {}
                    myD['retired_annotator'] = 'Not_Found'
                    self._connectStatusDB()
                    if self._statusDB:
                        rtList = self._statusDB.getRetiredAnnotatorInitials()
                        if rtList:
                            myD['retired_annotator'] = "', '".join(rtList)
                        #
                    #
                    tableMap['sql'] = sql_selection + ' ' + tableDef['sql_where_condition'] % myD
                else:
                    # assume that sql_variable is 'initials' & 'site'
                    tableMap['sql'] = sql_selection + ' ' + tableDef['sql_where_condition'] % self._userInfo
                #
            else:
                tableMap['sql'] = sql_selection + ' ' + tableDef['sql_where_condition']
            #
        else:
            tableMap['sql_selection'] = sql_selection
        #
        self.__tableContentMap[self.__tab_count_id + '_' + tableDef['table_id']] = tableMap

    def __processTabDefinition(self):
        """
        """
        secList = []
        if ('tab_definition_template' in self._conFigObj) and (self.__tab_def_id in self._conFigObj['tab_definition_template']):
            secList = self._conFigObj['tab_definition_template'][self.__tab_def_id]
        #
        if not secList:
            return
        #
        for secD in secList:
            if secD['template_type'] == 'html':
                self.__processHtmlTemplate(secD)
            #
        #

    def __processHtmlTemplate(self, secD):
        """
        """
        myParaD = self.__getPreProcessParameter(secD)
        page = self.getPageText(page_id=secD['template_id'], paraD=myParaD)
        if page:
            if not secD['section_id'] in self.__resultMap:
                self.__resultMap[secD['section_id']] = []
            #
            self.__resultMap[secD['section_id']].append(page)
        #

    def __getPreProcessParameter(self, secD):
        """
        """
        template_id = secD['template_id']
        paraD = {}
        key = self.__tab_def_id + ',' + template_id
        if ('tab_preprocess_mapping' not in self._conFigObj) or (key not in self._conFigObj['tab_preprocess_mapping']):
            return paraD
        #
        myD = {}
        if ('count_type' in secD) and secD['count_type'] and ('count_variables' in secD) and secD['count_variables'] and \
           ('count_template' in secD) and secD['count_template']:
            if self.__tab_count_id not in self.__countMap:
                self.__countMap[self.__tab_count_id] = {}
            #
            itemList = secD['count_variables'].split(',')
            self.__countMap[self.__tab_count_id][secD['count_type']] = itemList
            #
            tmplt = self._getPageTemplate(secD['count_template'])
            text = ''
            for item in itemList:
                if text:
                    text += ', '
                #
                text += tmplt % {'type': secD['count_type'], 'variable': item, 'tab_id': self.__tab_count_id}
            #
            myD['counts'] = text
        #
        for paraMap in self._conFigObj['tab_preprocess_mapping'][key]:
            if myD:
                paraD[paraMap['variable']] = self._preProcessParameterMap(paraMap['value'], myD)
            else:
                paraD[paraMap['variable']] = paraMap['value']
            #
        #
        return paraD

    def _get_entry_by_sg_center_enum_list(self):
        """
        """
        list = []  # pylint: disable=redefined-builtin
        cICommon = ConfigInfoAppCommon(self._siteId)
        sg_center_file = cICommon.get_sg_center_file_path()
        if os.access(sg_center_file, os.F_OK):
            cifObj = mmCIFUtil(filePath=sg_center_file)
            dlist = cifObj.GetValue('pdbx_SG_project')
            list.append(['', ''])
            for ldir in dlist:
                list.append([ldir['full_name_of_center'], ldir['full_name_of_center']])
            #
        #
        return list



##
# File:  MetaDataEditor.py
# Date:  23-Feb-2018
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2018 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import copy
import json
import multiprocessing
import os
import sys

from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass
from wwpdb.io.file.mmCIFUtil import mmCIFUtil
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
#


class MetaDataEditor(BaseClass):
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(MetaDataEditor, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        #
        self.__task = str(self._reqObj.getValue("task"))
        self.__dataType = str(self._reqObj.getValue("data_type"))
        #
        self.__statusCode = "ok"
        self.__errorMessage = ""
        self.__processMessage = ""
        self.__pickleFile = "metaData"
        self.__pickleObj = {}

    def setLogHandle(self, log=sys.stderr):
        """  Reset the stream for logging output.
        """
        try:
            self._lfh = log
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False
        #

    def ProcessForm(self):
        """
        """
        try:
            getattr(self, "%s" % self.__task)()
        except:  # noqa: E722 pylint: disable=bare-except
            if self.__errorMessage:
                self.__errorMessage += "\n"
            #
            self.__errorMessage += "Running process '" + self.__task + "' failed."
        #
        self.__pickleObj["statuscode"] = self.__statusCode
        self.__pickleObj["selected_datatype"] = self.__dataType
        for msgType in ("error", "message"):
            if msgType in self.__pickleObj:
                del self.__pickleObj[msgType]
            #
        #
        if self.__errorMessage:
            self.__pickleObj["error"] = self.__errorMessage
        #
        if self.__processMessage:
            self.__pickleObj["message"] = self.__processMessage
        #
        self._dumpPickle(self.__pickleFile, self.__pickleObj)
        #
        return True

    def GetFormProcessingResult(self):
        """
        """
        self.__pickleObj = self._loadPickle(self.__pickleFile)
        if ("statuscode" in self.__pickleObj) and self.__pickleObj["statuscode"]:
            self.__statusCode = self.__pickleObj["statuscode"]
        #
        if ("error" in self.__pickleObj) and self.__pickleObj["error"]:
            self.__errorMessage = self.__pickleObj["error"]
        #
        myD = {}
        myD["statuscode"] = self.__statusCode
        myD["statustext"] = self.__errorMessage
        if (self.__task == "get_entry") or (self.__task == "get_value"):
            self.__dataType = ""
            if ("selected_datatype" in self.__pickleObj) and self.__pickleObj["selected_datatype"]:
                self.__dataType = self.__pickleObj["selected_datatype"]
            #
            entryList = []
            if ("all_entry_ids" in self.__pickleObj) and self.__pickleObj["all_entry_ids"]:
                entryList = self.__pickleObj["all_entry_ids"]
            #
            htmlcontent = ""
            for entry_id in entryList:
                htmlcontent += self.__depictEntry(entry_id)
            #
            if htmlcontent:
                myD["htmlcontent"] = htmlcontent
            else:
                myD["statuscode"] = "failed"
                if not myD["statustext"]:
                    myD["statustext"] = "Load data failed."
                #
            #
        elif ("message" in self.__pickleObj) and self.__pickleObj["message"]:
            if myD["statustext"]:
                myD["statustext"] += "\n\n"
            #
            myD["statustext"] += self.__pickleObj["message"]
        #
        return myD

    def get_entry(self):
        """
        """
        self._removePickle(self.__pickleFile)
        #
        entry_ids = str(self._reqObj.getValue("entry_ids"))
        if not entry_ids:
            self.__errorMessage = "No Entry IDs defined."
            self.__statusCode = "failed"
            return
        #
        statusDB = StatusDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        self.__errorMessage, selectedEntryList = statusDB.getEntryIdListFromInputIdString(entry_ids)
        if not selectedEntryList:
            self.__statusCode = "failed"
            return
        #
        self.__all_data_type = str(self._reqObj.getValue("all_data_type"))  # pylint: disable=attribute-defined-outside-init
        all_data_type = self.__all_data_type.split(",")
        #
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runGetEntriesInfo")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=selectedEntryList, numProc=numProc, numResults=1)
        #
        validEntryList = []
        for entry_id in selectedEntryList:
            entryPickle = self._loadPickle(entry_id + "_MetaDataEditor")
            if ("error" in entryPickle) and entryPickle["error"]:
                if self.__errorMessage:
                    self.__errorMessage += "\n"
                #
                self.__errorMessage += entryPickle["error"]
            elif ("file" in entryPickle) and entryPickle["file"] and ("info" in entryPickle) and entryPickle["info"]:
                infoD = {}
                infoD["file"] = entryPickle["file"]
                if ("pdbid" in entryPickle["info"]) and entryPickle["info"]["pdbid"]:
                    infoD["pdbid"] = entryPickle["info"]["pdbid"]
                #
                if ("_pdbx_database_status.status_code" in entryPickle["info"]) and entryPickle["info"]["_pdbx_database_status.status_code"]:
                    infoD["status_code"] = entryPickle["info"]["_pdbx_database_status.status_code"]
                #
                if ("_pdbx_database_status.pdbx_annotator" in entryPickle["info"]) and entryPickle["info"]["_pdbx_database_status.pdbx_annotator"]:
                    infoD["annotator"] = entryPickle["info"]["_pdbx_database_status.pdbx_annotator"]
                #
                dataD = {}
                for data_type in all_data_type:
                    if (data_type in entryPickle["info"]) and entryPickle["info"][data_type]:
                        dataD[data_type] = entryPickle["info"][data_type]
                    #
                #
                infoD["original_data"] = dataD
                infoD["current_data"] = copy.deepcopy(dataD)
                self.__pickleObj[entry_id] = infoD
                validEntryList.append(entry_id)
            #
            self._removePickle(entry_id + "_MetaDataEditor")
        #
        if not validEntryList:
            self.__statusCode = "failed"
        else:
            self.__pickleObj["all_entry_ids"] = validEntryList
        #

    def get_value(self):
        """
        """
        self.__pickleObj = self._loadPickle(self.__pickleFile)

    def save_value(self):
        """
        """
        self.__pickleObj = self._loadPickle(self.__pickleFile)
        entryList = self.__getEntryList()
        if not entryList:
            return
        #
        for entry_id in entryList:
            value = str(self._reqObj.getValue("current_" + entry_id))
            # updated = False
            if (entry_id in self.__pickleObj) and self.__pickleObj[entry_id] and ("current_data" in self.__pickleObj[entry_id]) and \
               self.__pickleObj[entry_id]["current_data"]:
                self.__pickleObj[entry_id]["current_data"][self.__dataType] = value
                if self.__processMessage:
                    self.__processMessage += "\n"
                #
                self.__processMessage += "Value saved for " + entry_id + "."
            else:
                if self.__errorMessage:
                    self.__errorMessage += "\n"
                #
                self.__errorMessage += "Saving value failed for " + entry_id + "."
            #
        #

    def update_file(self):
        """
        """
        self.__pickleObj = self._loadPickle(self.__pickleFile)
        entryList = self.__getEntryList()
        if not entryList:
            return
        #
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runUpdateEntriesInfo")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=entryList, numProc=numProc, numResults=1)
        #
        for entry_id in entryList:
            entryPickle = self._loadPickle(entry_id + "_MetaDataEditor")
            if ("error" in entryPickle) and entryPickle["error"]:
                if self.__errorMessage:
                    self.__errorMessage += "\n"
                #
                self.__errorMessage += entryPickle["error"]
            elif ("message" in entryPickle) and entryPickle["message"]:
                if self.__processMessage:
                    self.__processMessage += "\n"
                #
                self.__processMessage += entryPickle["message"]
            #
            self._removePickle(entry_id + "_MetaDataEditor")
        #

    def runGetEntriesInfo(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runGetEntryInfo(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def runUpdateEntriesInfo(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runUpdateEntryInfo(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __runGetEntryInfo(self, entry_id):
        """
        """
        entryPickle = {}
        message, modelFile = self._getExistingArchiveFile(entry_id, "model", "pdbx", "latest")
        if message:
            entryPickle["error"] = message
            self._dumpPickle(entry_id + "_MetaDataEditor", entryPickle)
            return
        #
        entryPickle["file"] = modelFile
        message, info = self.__getEntryInfoFromModelFile(entry_id, modelFile)
        if message:
            entryPickle["error"] = message
            self._dumpPickle(entry_id + "_MetaDataEditor", entryPickle)
            return
        #
        entryPickle["info"] = info
        self._dumpPickle(entry_id + "_MetaDataEditor", entryPickle)

    def __getEntryInfoFromModelFile(self, entry_id, inputFile):
        """
        """
        jsonFile = entry_id + "_MetaDataEditor.json"
        logFile = "MetaDataEditor_" + entry_id + ".log"
        clogFile = "MetaDataEditor_command_" + entry_id + ".log"
        extraOption = " -data_type '" + self.__all_data_type + ",_pdbx_database_status.status_code,_pdbx_database_status.pdbx_annotator,pdbid' "
        cmd = self._getCmd("${BINPATH}/GetMetaDataInfo", inputFile, jsonFile, logFile, clogFile, extraOption)
        self._runCmd(cmd)
        #
        msg = self._getLogMessage(os.path.join(self._sessionPath, logFile))
        cmsg = self._getLogMessage(os.path.join(self._sessionPath, clogFile))
        if cmsg:
            if msg:
                msg += "\n"
            #
            msg += cmsg
        #
        if msg:
            return msg, {}
        #
        if os.access(os.path.join(self._sessionPath, jsonFile), os.F_OK):
            with open(os.path.join(self._sessionPath, jsonFile), "r") as f:
                jsonObj = json.load(f)
            #
            return "", jsonObj
        #
        return "Get entry information failed.", {}

    def __depictEntry(self, entry_id):
        """
        """
        if not self.__dataType:
            return ""
        #
        myD = {}
        myD["entry_id"] = entry_id
        for item in ("original_data", "current_data", "pdbid", "status_code", "annotator"):
            myD[item] = ""
        #
        if (entry_id in self.__pickleObj) and self.__pickleObj[entry_id]:
            for item in ("pdbid", "status_code", "annotator"):
                if (item in self.__pickleObj[entry_id]) and self.__pickleObj[entry_id][item]:
                    myD[item] = self.__pickleObj[entry_id][item]
                #
            #
            for item in ("original_data", "current_data"):
                if (item not in self.__pickleObj[entry_id]) or (not self.__pickleObj[entry_id][item]):
                    continue
                #
                if (self.__dataType not in self.__pickleObj[entry_id][item]) or (not self.__pickleObj[entry_id][item][self.__dataType]):
                    continue
                #
                value = self.__pickleObj[entry_id][item][self.__dataType]
                if item == "original_data":
                    value = value.replace('"', '\\"')
                #
                myD[item] = value
            #
        #
        return self._processTemplate("editing_entry_tmplt.html", myD)

    def __getEntryList(self):
        """
        """
        entryList = self._reqObj.getValueList('entry')
        if not entryList:
            self.__statusCode = "failed"
            self.__errorMessage = "No Entry selected."
        #
        return entryList

    def __runUpdateEntryInfo(self, entry_id):
        """
        """
        entryPickle = {}
        #
        if (entry_id not in self.__pickleObj) or (not self.__pickleObj[entry_id]) or ("current_data" not in self.__pickleObj[entry_id]) or \
           (not self.__pickleObj[entry_id]["current_data"]) or ("file" not in self.__pickleObj[entry_id]) or (not self.__pickleObj[entry_id]["file"]):
            entryPickle["error"] = "Updating " + entry_id + " failed."
            self._dumpPickle(entry_id + "_MetaDataEditor", entryPickle)
            return
        #
        dataFile = self.__writeDataCifFile(entry_id, self.__pickleObj[entry_id]["current_data"])
        #
        message, updatedModelFile = self.__updateModelFile(entry_id, self.__pickleObj[entry_id]["file"], dataFile)
        if message:
            entryPickle["error"] = "Updating " + entry_id + " failed:\n" + message
            self._dumpPickle(entry_id + "_MetaDataEditor", entryPickle)
            return
        #
        archiveModelFile = self._findArchiveFileName(entry_id, 'model', 'pdbx', 'next')
        message = self._copyFileUtil(updatedModelFile, archiveModelFile)
        if message:
            entryPickle["error"] = "Updating " + entry_id + " failed:\n" + message
        else:
            entryPickle["message"] = entry_id + " updated."
        #
        self._dumpPickle(entry_id + "_MetaDataEditor", entryPickle)

    def __writeDataCifFile(self, entry_id, dataInfo):
        """
        """
        dataFile = entry_id + "_data.cif"
        cifObj = mmCIFUtil()
        cifObj.AddBlock(entry_id)
        categoryMap = {}
        for key, v in dataInfo.items():
            value = v
            if not value:
                value = "?"
            #
            cList = key.split(".")
            if cList[0][1:] in categoryMap:
                categoryMap[cList[0][1:]][0].append(cList[1])
                categoryMap[cList[0][1:]][1][0].append(value)
            else:
                categoryMap[cList[0][1:]] = [[cList[1]], [[value]]]
            #
        #
        for key, values in categoryMap.items():
            cifObj.AddCategory(key, values[0])
            cifObj.InsertData(key, values[1])
        #
        cifObj.WriteCif(outputFilePath=os.path.join(self._sessionPath, dataFile))
        return dataFile

    def __updateModelFile(self, entry_id, modelFile, dataFile):
        """
        """
        updatedModelFile = entry_id + "_MetaDataEditor.cif"
        # jsonFile = entry_id + "_MetaDataEditor.json"
        logFile = "MetaDataEditor_" + entry_id + ".log"
        clogFile = "MetaDataEditor_command_" + entry_id + ".log"
        extraOption = " -data_file " + dataFile + " "
        cmd = self._getCmd("${BINPATH}/MergeMetaDataInfo", modelFile, updatedModelFile, logFile, clogFile, extraOption)
        self._runCmd(cmd)
        #
        msg = self._getLogMessage(os.path.join(self._sessionPath, logFile))
        cmsg = self._getLogMessage(os.path.join(self._sessionPath, clogFile))
        if cmsg:
            if msg:
                msg += "\n"
            #
            msg += cmsg
        #
        if msg:
            return msg, ''
        #
        if os.access(os.path.join(self._sessionPath, updatedModelFile), os.F_OK):
            return "", os.path.join(self._sessionPath, updatedModelFile)
        #
        return "Merge metadata information failed.", ""



##
# File:  LigandFinder.py
# Date:  29-Mar-2020
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2020 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import os
import sys

from wwpdb.apps.workmanager.db_access.ContentDbApi import ContentDbApi
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass


class LigandFinder(BaseClass):
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(LigandFinder, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        #
        self.__statusDB = StatusDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        self.__contentDB = ContentDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        #
        self.__entryList = self.__statusDB.getEntryListForGroup(groupids=[self._reqObj.getValue("identifier")])

    def getLigandInfo(self):
        """
        """
        if not self.__entryList:
            return {}, "No entry found."
        #
        entryIdList = []
        for entry in self.__entryList:
            if ("dep_set_id" in entry) and entry["dep_set_id"]:
                entryIdList.append(entry["dep_set_id"])
            #
        #
        if not entryIdList:
            return {}, "No entry found."
        #
        ligList = self.__contentDB.getLigandIdList(entryIdList=entryIdList)
        if not ligList:
            return {}, "No ligand found."
        #
        ligMap = {}
        for ligDir in ligList:
            if ("Structure_ID" not in ligDir) or (not ligDir["Structure_ID"]) or ("comp_id" not in ligDir) or \
               (not ligDir["comp_id"]) or (ligDir["comp_id"].upper() == "HOH"):
                continue
            #
            if ligDir["Structure_ID"] in ligMap:
                ligMap[ligDir["Structure_ID"]].append(ligDir["comp_id"].upper())
            else:
                ligMap[ligDir["Structure_ID"]] = [ligDir["comp_id"].upper()]
            #
        #
        if not ligMap:
            return {}, "No ligand found."
        #
        return ligMap, "OK"


if __name__ == '__main__':
    from wwpdb.utils.session.WebRequest import InputRequest
    from wwpdb.utils.config.ConfigInfo import ConfigInfo
    siteId = os.getenv("WWPDB_SITE_ID")
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002010")
    myReqObj.setValue("sessionid", "79ae0371518fe08d9d3e1fb8fdd001055632d48a")
    ligFinder = LigandFinder(reqObj=myReqObj, verbose=False, log=sys.stderr)
    retMap, errMsg = ligFinder.getLigandInfo()
    print(errMsg)
    print(retMap)



##
# File:  ManageSite.py
# Date:  20-Nov-2019
# Updates:
##
"""
Tool for managing the OneDep workflow manager user database.

This tool allows for queryng adding/removing sites, and dumping restoring users
"""
__docformat__ = "restructuredtext en"
__author__ = "Ezra Peisach"
__email__ = "peisach@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import argparse
import sys

from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi


class UserManager(object):
    def __init__(self, siteId=None):
        self.__siteId = siteId
        self.__statusDB = StatusDbApi(siteId=self.__siteId, verbose=True)

    def listSites(self, verbose):
        sites = self.__statusDB.getSites()
        sdata = {}
        for s in sites:
            site = s['site']
            if site == 'ALL':
                continue
            if site not in sdata:
                sdata[site] = {}
            sdata[site][s['code']] = {'da_group_id': s['da_group_id'],
                                      'group_name' : s['group_name']}

        for s in sdata:
            if verbose:
                print("Site: {}".format(s))
                for code in ['LANN', 'ANN']:
                    if code in sdata[s]:
                        d = sdata[s][code]
                        print("     {:4s} group: {:2d}  name: {}".format(code, d['da_group_id'], d['group_name']))
            else:
                print(s)

    def listSiteUsers(self, site):
        users = self.__statusDB.getSiteUser(site)

        for u in users:
            print('{:4s} uname: {:6s} Name: {:30s}   initials {:6s} active: {:1d}  pass: {:s}'.format(u['code'],
                                                                                                      u['user_name'],
                                                                                                      "%s %s" % (u['first_name'], u['last_name']),
                                                                                                      u['initials'],
                                                                                                      u['active'],
                                                                                                      u['password']
                                                                                                      ))

    def addSite(self, site, lead, email, first, last):
        status = self.__statusDB. addSite(site, lead, email, first, last)
        if status != 'OK':
            print("Problem adding site - might already exist")

    def deleteSite(self, site):
        status = self.__statusDB.deleteSite(site)
        if status != 'OK':
            print(status)


def main():
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(help='sub-command help')
    sub_a = subparsers.add_parser('list-sites', help='list installed webfe packages')
    sub_a.add_argument("-v", "--verbose", action='store_true', help='Display additional info for site')
    sub_a.set_defaults(func="list")

    sub_b = subparsers.add_parser('list-users', help='list users in WFM db for site')
    sub_b.add_argument("-s", "--wfm-site", help='WFM site id to list', required=True)
    sub_b.set_defaults(func="list_users")

    sub_c = subparsers.add_parser('add-site', help='add a new site')
    sub_c.add_argument("-s", "--wfm-site", help='WFM site id to list', required=True)
    sub_c.add_argument("--lead-user", help='Lead user for assignments', required=True)
    sub_c.add_argument("--email", help='email of lead user', required=True)
    sub_c.add_argument("--first-name", help='first name of lead user', required=True)
    sub_c.add_argument("--last-name", help='last name of lead user', required=True)
    sub_c.set_defaults(func="add_site")

    sub_d = subparsers.add_parser('delete-site', help='add a new site')
    sub_d.add_argument("-s", "--wfm-site", help='WFM site id to list', required=True)
    sub_d.set_defaults(func="delete_site")

    args = parser.parse_args()

    if 'func' not in args:
        parser.print_usage()
        sys.exit(1)

    print(args)

    um = UserManager()

    if args.func == 'list':
        um.listSites(args.verbose)
    elif args.func == 'list_users':
        um.listSiteUsers(site=args.wfm_site)
    elif args.func == 'add_site':
        um.addSite(site=args.wfm_site, lead=args.lead_user, email=args.email,
                   first=args.first_name, last=args.last_name)
    elif args.func == 'delete_site':
        um.deleteSite(site=args.wfm_site)


if __name__ == '__main__':
    main()



##
# File:  LogFileUtil.py
# Date:  21-June-2015
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import os
import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi


class LogFileUtil(object):
    """
    """
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__verbose = verbose
        self.__lfh = log
        self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
        self.__cI = ConfigInfo(self.__siteId)
        self.__depositionid = str(self.__reqObj.getValue("identifier"))
        self.__instanceid = str(self.__reqObj.getValue("instance"))
        self.__classid = str(self.__reqObj.getValue("classID"))
        self.__taskid = str(self.__reqObj.getValue("taskID"))
        self.__reference = str(self.__reqObj.getValue("reference"))
        self.__logPath = os.path.join(self.__cI.get('SITE_ARCHIVE_STORAGE_PATH'), 'archive', self.__depositionid, 'log')

    def getLogFile(self):
        """
        """
        if self.__taskid:
            filename = self.__reference + '_' + self.__depositionid + '_' + self.__classid + '_' + self.__instanceid \
                + '_' + self.__taskid + '.log'
        else:
            statusDB = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
            classInfo = statusDB.getWfClassByID(classID=self.__classid)
            filename = 'WFE_' + self.__depositionid + classInfo['class_file'][:-4] + '.log'
        #
        filePath = os.path.join(self.__logPath, filename)
        if os.access(filePath, os.F_OK):
            return filePath
        #
        return ''
